@@ -24,15 +24,11 @@
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
-
-#include "c_lib.h"
-#include "c_private.h"
-
 #include "csync_exclude.h"
 
-#include "../version.h"
 #include "common/filesystembase.h"
 #include "common/utility.h"
+#include "common/version.h"
 
 #include <QString>
 #include <QFileInfo>
@@ -104,7 +100,7 @@ OCSYNC_EXPORT void csync_exclude_expand_escapes(QByteArray &input)
  * @param file_name filename
  * @return true if file is reserved, false otherwise
  */
-OCSYNC_EXPORT bool csync_is_windows_reserved_word(const QStringRef &filename)
+OCSYNC_EXPORT bool csync_is_windows_reserved_word(QStringView filename)
 {
     size_t len_filename = filename.size();
 
@@ -147,10 +143,10 @@ OCSYNC_EXPORT bool csync_is_windows_reserved_word(const QStringRef &filename)
     return false;
 }
 
-static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QStringRef &path, bool excludeConflictFiles)
+static CSYNC_EXCLUDE_TYPE _csync_excluded_common(QStringView path, bool excludeConflictFiles)
 {
     /* split up the path */
-    QStringRef bname(path);
+    QStringView bname(path);
     int lastSlash = path.lastIndexOf(QLatin1Char('/'));
     if (lastSlash >= 0) {
         bname = path.mid(lastSlash + 1);
@@ -200,23 +196,16 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QStringRef &path, bool ex
 
     // Filter out characters not allowed in a filename on windows
     // see https://docs.microsoft.com/en-us/windows/desktop/fileio/naming-a-file
-    for (auto p : path) {
-        ushort c = p.unicode();
-        if (c < 32) {
+    for (auto c : path) {
+        if (c.unicode() < 32) {
             return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
         }
-        switch (c) {
-        case '\\':
-        case ':':
-        case '?':
-        case '*':
-        case '"':
-        case '>':
-        case '<':
-        case '|':
+        if (std::find_if(
+                OCC::FileSystem::IllegalFilenameCharsWindows.begin(), OCC::FileSystem::IllegalFilenameCharsWindows.end(), [c](const auto illegal) {
+                    return c == illegal;
+                })
+            != OCC::FileSystem::IllegalFilenameCharsWindows.end()) {
             return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
-        default:
-            break;
         }
     }
 #endif
@@ -238,7 +227,7 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QStringRef &path, bool ex
 using namespace OCC;
 
 ExcludedFiles::ExcludedFiles()
-    : _clientVersion(MIRALL_VERSION_MAJOR, MIRALL_VERSION_MINOR, MIRALL_VERSION_PATCH)
+    : _clientVersion(OCC::Version::version())
 {
     // Windows used to use PathMatchSpec which allows *foo to match abc/deffoo.
     _wildcardsMatchSlash = Utility::isWindows();
@@ -277,7 +266,7 @@ void ExcludedFiles::setWildcardsMatchSlash(bool onoff)
     prepare();
 }
 
-void ExcludedFiles::setClientVersion(ExcludedFiles::Version version)
+void ExcludedFiles::setClientVersion(const QVersionNumber &version)
 {
     _clientVersion = version;
 }
@@ -321,7 +310,7 @@ bool ExcludedFiles::versionDirectiveKeepNextLine(const QByteArray &directive) co
     if (argVersions.size() != 3)
         return true;
 
-    auto argVersion = std::make_tuple(argVersions[0].toInt(), argVersions[1].toInt(), argVersions[2].toInt());
+    const auto argVersion = QVersionNumber(argVersions[0].toInt(), argVersions[1].toInt(), argVersions[2].toInt());
     if (op == "<=")
         return _clientVersion <= argVersion;
     if (op == "<")
@@ -335,18 +324,19 @@ bool ExcludedFiles::versionDirectiveKeepNextLine(const QByteArray &directive) co
     return true;
 }
 
-bool ExcludedFiles::isExcluded(
-    const QString &filePath,
-    const QString &basePath,
-    bool excludeHidden) const
+bool ExcludedFiles::isExcluded(QStringView filePath, QStringView basePath, bool excludeHidden) const
 {
     if (!FileSystem::isChildPathOf(filePath, basePath)) {
         // Mark paths we're not responsible for as excluded...
         return true;
     }
 
-    const QFileInfo fileInfo(filePath);
+    const QFileInfo fileInfo(filePath.toString());
     if (!fileInfo.exists()) {
+        if (excludeHidden && fileInfo.fileName().startsWith(QLatin1Char('.'))) {
+            return true;
+        }
+        // recursively call isExcluded with the path of non-existing file's parent directory
         return isExcluded(fileInfo.path(), basePath, excludeHidden);
     }
 
@@ -360,7 +350,7 @@ bool ExcludedFiles::isExcluded(
             }
 
             // Get the parent path
-            fi = { fi.path() };
+            fi = QFileInfo{fi.path()};
         }
     }
     ItemType type = ItemTypeFile;
@@ -370,14 +360,14 @@ bool ExcludedFiles::isExcluded(
     return isExcludedRemote(filePath, basePath, excludeHidden, type);
 }
 
-bool ExcludedFiles::isExcludedRemote(const QString &filePath, const QString &basePath, bool excludeHidden, ItemType type) const
+bool ExcludedFiles::isExcludedRemote(QStringView filePath, QStringView basePath, bool excludeHidden, ItemType type) const
 {
     if (!FileSystem::isChildPathOf(filePath, basePath)) {
         // Mark paths we're not responsible for as excluded...
         return true;
     }
 
-    auto relativePath = filePath.midRef(basePath.size());
+    auto relativePath = filePath.mid(basePath.size());
     if (relativePath.endsWith(QLatin1Char('/'))) {
         relativePath.chop(1);
     }
@@ -393,7 +383,7 @@ bool ExcludedFiles::isExcludedRemote(const QString &filePath, const QString &bas
     return fullPatternMatch(relativePath, type) != CSYNC_NOT_EXCLUDED;
 }
 
-CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QStringRef &path, ItemType filetype) const
+CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(QStringView path, ItemType filetype) const
 {
     auto match = _csync_excluded_common(path, _excludeConflictFiles);
     if (match != CSYNC_NOT_EXCLUDED)
@@ -403,7 +393,7 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QStringRef &path,
 
     // Check the bname part of the path to see whether the full
     // regex should be run.
-    QStringRef bnameStr(path);
+    QStringView bnameStr(path);
     int lastSlash = path.lastIndexOf(QLatin1Char('/'));
     if (lastSlash >= 0) {
         bnameStr = path.mid(lastSlash + 1);
@@ -424,7 +414,7 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QStringRef &path,
     }
 
     // third capture: full path matching is triggered
-    QStringRef pathStr = path;
+    QStringView pathStr = path;
 
     if (filetype == ItemTypeDirectory) {
         m = _fullTraversalRegexDir.match(pathStr);
@@ -441,7 +431,7 @@ CSYNC_EXCLUDE_TYPE ExcludedFiles::traversalPatternMatch(const QStringRef &path,
     return CSYNC_NOT_EXCLUDED;
 }
 
-CSYNC_EXCLUDE_TYPE ExcludedFiles::fullPatternMatch(const QStringRef &p, ItemType filetype) const
+CSYNC_EXCLUDE_TYPE ExcludedFiles::fullPatternMatch(QStringView p, ItemType filetype) const
 {
     auto match = _csync_excluded_common(p, _excludeConflictFiles);
     if (match != CSYNC_NOT_EXCLUDED)
