@@ -20,15 +20,10 @@
  */
 
 #include <errno.h>
-#include <fcntl.h>
-#include <stdio.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-
-#include "windows.h"
 
 #include "common/filesystembase.h"
 #include "common/utility.h"
+#include "common/utility_win.h"
 #include "csync.h"
 #include "vio/csync_vio_local.h"
 
@@ -52,7 +47,7 @@ struct csync_vio_handle_t
 
 csync_vio_handle_t *csync_vio_local_opendir(const QString &name)
 {
-    QScopedPointer<csync_vio_handle_t> handle(new csync_vio_handle_t {});
+    std::unique_ptr<csync_vio_handle_t> handle(new csync_vio_handle_t{});
 
     // the file wildcard has to be attached
     QString dirname = OCC::FileSystem::longWinPath(name + QLatin1String("/*"));
@@ -73,7 +68,7 @@ csync_vio_handle_t *csync_vio_local_opendir(const QString &name)
 
     dirname.chop(1); // remove the *
     handle->path = std::move(dirname);
-    return handle.take();
+    return handle.release();
 }
 
 int csync_vio_local_closedir(csync_vio_handle_t *dhandle)
