@@ -16,24 +16,18 @@
 #include <QtWidgets>
 
 #include "QProgressIndicator.h"
+
 #include "account.h"
 #include "accountmanager.h"
 #include "accountstate.h"
 #include "activitywidget.h"
 #include "commonstrings.h"
-#include "configfile.h"
-#include "folder.h"
-#include "folderman.h"
 #include "issueswidget.h"
-#include "logger.h"
 #include "notificationconfirmjob.h"
 #include "notificationwidget.h"
-#include "ocsjob.h"
 #include "openfilemanager.h"
-#include "owncloudpropagator.h"
 #include "protocolwidget.h"
 #include "servernotificationhandler.h"
-#include "syncfileitem.h"
 #include "syncresult.h"
 #include "theme.h"
 
@@ -45,6 +39,9 @@
 
 #include <climits>
 
+using namespace std::chrono;
+using namespace std::chrono_literals;
+
 // time span in milliseconds which has to be between two
 // refreshes of the notifications
 #define NOTIFICATION_REQUEST_FREE_PERIOD 15000
@@ -59,7 +56,7 @@ ActivityWidget::ActivityWidget(QWidget *parent)
     _ui->setupUi(this);
 
     _model = new ActivityListModel(this);
-    _sortModel = new SignalledQSortFilterProxyModel(this);
+    _sortModel = new Models::SignalledQSortFilterProxyModel(this);
     _sortModel->setSourceModel(_model);
     _ui->_activityList->setModel(_sortModel);
     _sortModel->setSortRole(Models::UnderlyingDataRole);
@@ -86,19 +83,19 @@ ActivityWidget::ActivityWidget(QWidget *parent)
     connect(_model, &ActivityListModel::activityJobStatusCode,
         this, &ActivityWidget::slotAccountActivityStatus);
 
-    connect(AccountManager::instance(), &AccountManager::accountRemoved, this, [this](const AccountStatePtr &ast) {
+    connect(AccountManager::instance(), &AccountManager::accountRemoved, this, [this](AccountStatePtr ast) {
         if (_accountsWithoutActivities.remove(ast->account()->displayName())) {
             showLabels();
         }
 
         for (const auto widget : qAsConst(_widgetForNotifId)) {
-            if (widget->activity().uuid() == ast->account()->uuid()) {
+            if (widget->activity().accountUuid() == ast->account()->uuid()) {
                 scheduleWidgetToRemove(widget);
             }
         }
     });
 
-    connect(_model, &QAbstractItemModel::modelReset, this, &ActivityWidget::dataChanged);
+    connect(_model, &ActivityListModel::activityJobStatusCode, this, &ActivityWidget::dataChanged);
     connect(_ui->_activityList, &QListView::customContextMenuRequested, this, &ActivityWidget::slotItemContextMenu);
     header->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(header, &QListView::customContextMenuRequested, header, [header, this] {
@@ -110,9 +107,8 @@ ActivityWidget::ActivityWidget(QWidget *parent)
     connect(_ui->_filterButton, &QAbstractButton::clicked, this, [this] {
         ProtocolWidget::showFilterMenu(_ui->_filterButton, _sortModel, static_cast<int>(ActivityListModel::ActivityRole::Account), tr("Account"));
     });
-    connect(_sortModel, &SignalledQSortFilterProxyModel::filterChanged, [this]() {
-        _ui->_filterButton->setText(CommonStrings::filterButtonText(_sortModel->filterRegExp().isEmpty() ? 0 : 1));
-    });
+    connect(_sortModel, &Models::SignalledQSortFilterProxyModel::filterChanged, this,
+        [this]() { _ui->_filterButton->setText(CommonStrings::filterButtonText(_sortModel->filterRegularExpression().pattern().isEmpty() ? 0 : 1)); });
 
     connect(&_removeTimer, &QTimer::timeout, this, &ActivityWidget::slotCheckToCleanWidgets);
     _removeTimer.setInterval(1000);
@@ -123,12 +119,12 @@ ActivityWidget::~ActivityWidget()
     delete _ui;
 }
 
-void ActivityWidget::slotRefreshActivities(AccountState *ptr)
+void ActivityWidget::slotRefreshActivities(const AccountStatePtr &ptr)
 {
     _model->slotRefreshActivity(ptr);
 }
 
-void ActivityWidget::slotRefreshNotifications(AccountState *ptr)
+void ActivityWidget::slotRefreshNotifications(const AccountStatePtr &ptr)
 {
     // start a server notification handler if no notification requests
     // are running
@@ -159,13 +155,13 @@ void ActivityWidget::showLabels()
     t.clear();
     QSetIterator<QString> i(_accountsWithoutActivities);
     while (i.hasNext()) {
-        t.append(tr("<br/>Account %1 does not have activities enabled.").arg(i.next()));
+        t.append(tr("<br/>%1 does not provide activities.").arg(i.next()));
     }
     _ui->_bottomLabel->setTextFormat(Qt::RichText);
     _ui->_bottomLabel->setText(t);
 }
 
-void ActivityWidget::slotAccountActivityStatus(AccountState *ast, int statusCode)
+void ActivityWidget::slotAccountActivityStatus(AccountStatePtr ast, int statusCode)
 {
     if (!(ast && ast->account())) {
         return;
@@ -206,7 +202,7 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
         return;
     }
     // compute the count to display later
-    QHash<QString, int> accNotified;
+    QMap<QString, int> accNotified;
 
     // Whether a new notification widget was added to the notificationLayout.
     bool newNotificationShown = false;
@@ -244,7 +240,7 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
         // will repeat the gui log.
 
         // after one hour, clear the gui log notification store
-        if (_guiLogTimer.elapsed() > 60 * 60 * 1000) {
+        if (_guiLogTimer.elapsed() > duration_cast<microseconds>(1h).count()) {
             _guiLoggedNotifications.clear();
         }
         if (!_guiLoggedNotifications.contains(activity.id())) {
@@ -258,12 +254,12 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
     // check if there are widgets that have no corresponding activity from
     // the server any more. Collect them in a list
 
-    const auto accId = list.first().uuid();
-    QList<Activity::Identifier> strayCats;
+    const auto accId = list.first().accountUuid();
+    QList<QString> strayCats;
     for (auto it = _widgetForNotifId.cbegin(); it != _widgetForNotifId.cend(); ++it) {
         bool found = false;
         // do not mark widgets of other accounts to delete.
-        if (it.value()->activity().uuid() != accId) {
+        if (it.value()->activity().accountUuid() != accId) {
             continue;
         }
 
@@ -300,7 +296,7 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
             msg = tr("%n notifications(s) for %1.", "", accNotified.begin().value()).arg(accNotified.begin().key());
         } else if (newGuiLogCount >= 2) {
             const auto acc1 = accNotified.begin();
-            const auto acc2 = acc1 + 1;
+            const auto acc2 = ++accNotified.begin();
             if (newGuiLogCount == 2) {
                 const int notiCount = acc1.value() + acc2.value();
                 msg = tr("%n notifications(s) for %1 and %2.", "", notiCount).arg(acc1.key(), acc2.key());
@@ -322,22 +318,36 @@ void ActivityWidget::slotSendNotificationRequest(const QString &accountName, con
     qCInfo(lcActivity) << "Server Notification Request " << verb << link << "on account" << accountName;
     NotificationWidget *theSender = qobject_cast<NotificationWidget *>(sender());
 
-    const QStringList validVerbs = { QStringLiteral("GET"),
-        QStringLiteral("PUT"),
-        QStringLiteral("POST"),
-        QStringLiteral("DELETE") };
+    const QList<QByteArray> validVerbs = {
+        QByteArrayLiteral("GET"),
+        QByteArrayLiteral("PUT"),
+        QByteArrayLiteral("POST"),
+        QByteArrayLiteral("DELETE")
+    };
 
     if (validVerbs.contains(verb)) {
-        AccountStatePtr acc = AccountManager::instance()->account(accountName);
-        if (acc) {
-            NotificationConfirmJob *job = new NotificationConfirmJob(acc->account());
-            QUrl l(link);
-            job->setLinkAndVerb(l, verb);
+        if (auto acc = AccountManager::instance()->account(accountName)) {
+            // TODO: host validation?
+            auto *job = new NotificationConfirmJob(acc->account(), QUrl(link), verb, this);
             job->setWidget(theSender);
-            connect(job, &AbstractNetworkJob::networkError,
-                this, &ActivityWidget::slotNotifyNetworkError);
-            connect(job, &NotificationConfirmJob::jobFinished,
-                this, &ActivityWidget::slotNotifyServerFinished);
+            connect(job, &NotificationConfirmJob::finishedSignal,
+                this, [job, this] {
+                    if (job->reply()->error() == QNetworkReply::NoError) {
+                        endNotificationRequest(job->widget(), job->ocsSuccess());
+                        qCInfo(lcActivity) << "Server Notification reply code" << job->ocsStatus();
+
+                        // if the notification was successful start a timer that triggers
+                        // removal of the done widgets in a few seconds
+                        // Add 200 millisecs to the predefined value to make sure that the timer in
+                        // widget's method readyToClose() has elapsed.
+                        if (job->ocsSuccess()) {
+                            scheduleWidgetToRemove(job->widget());
+                        }
+                    } else {
+                        endNotificationRequest(job->widget(), job->ocsSuccess());
+                        qCWarning(lcActivity) << "Server notify job failed with code " << job->ocsStatus();
+                    }
+                });
             job->start();
 
             // count the number of running notification requests. If this member var
@@ -349,43 +359,11 @@ void ActivityWidget::slotSendNotificationRequest(const QString &accountName, con
     }
 }
 
-void ActivityWidget::endNotificationRequest(NotificationWidget *widget, int replyCode)
+void ActivityWidget::endNotificationRequest(NotificationWidget *widget, bool success)
 {
     _notificationRequestsRunning--;
     if (widget) {
-        widget->slotNotificationRequestFinished(replyCode);
-    }
-}
-
-void ActivityWidget::slotNotifyNetworkError(QNetworkReply *reply)
-{
-    NotificationConfirmJob *job = qobject_cast<NotificationConfirmJob *>(sender());
-    if (!job) {
-        return;
-    }
-
-    int resultCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-
-    endNotificationRequest(job->widget(), resultCode);
-    qCWarning(lcActivity) << "Server notify job failed with code " << resultCode;
-}
-
-void ActivityWidget::slotNotifyServerFinished(const QString &reply, int replyCode)
-{
-    NotificationConfirmJob *job = qobject_cast<NotificationConfirmJob *>(sender());
-    if (!job) {
-        return;
-    }
-
-    endNotificationRequest(job->widget(), replyCode);
-    qCInfo(lcActivity) << "Server Notification reply code" << replyCode << reply;
-
-    // if the notification was successful start a timer that triggers
-    // removal of the done widgets in a few seconds
-    // Add 200 millisecs to the predefined value to make sure that the timer in
-    // widget's method readyToClose() has elapsed.
-    if (replyCode == OCS_SUCCESS_STATUS_CODE || replyCode == OCS_SUCCESS_STATUS_CODE_V2) {
-        scheduleWidgetToRemove(job->widget());
+        widget->slotNotificationRequestFinished(success);
     }
 }
 
@@ -428,8 +406,7 @@ void ActivityWidget::slotCheckToCleanWidgets()
 
         if (currentTime > t) {
             // found one to remove!
-            Activity::Identifier id = widget->activity().id();
-            _widgetForNotifId.remove(id);
+            _widgetForNotifId.remove(widget->activity().id());
             widget->deleteLater();
             it = _widgetsToRemove.erase(it);
         } else {
@@ -512,6 +489,9 @@ ActivitySettings::ActivitySettings(QWidget *parent)
 
     // We want the protocol be the default
     _tab->setCurrentIndex(1);
+
+    connect(AccountManager::instance(), &AccountManager::accountRemoved, this,
+        [this](const AccountStatePtr &accountStatePtr) { _timeSinceLastCheck.take(accountStatePtr); });
 }
 
 void ActivitySettings::setNotificationRefreshInterval(std::chrono::milliseconds interval)
@@ -565,7 +545,7 @@ void ActivitySettings::slotRemoveAccount(const AccountStatePtr &ptr)
     _activityWidget->slotRemoveAccount(ptr);
 }
 
-void ActivitySettings::slotRefresh(AccountState *ptr)
+void ActivitySettings::slotRefresh(AccountStatePtr ptr)
 {
     // QElapsedTimer isn't actually constructed as invalid.
     if (!_timeSinceLastCheck.contains(ptr)) {
@@ -591,7 +571,7 @@ void ActivitySettings::slotRefresh(AccountState *ptr)
 void ActivitySettings::slotRegularNotificationCheck()
 {
     for (const auto &a : AccountManager::instance()->accounts()) {
-        slotRefresh(a.data());
+        slotRefresh(a);
     }
 }
 
