@@ -934,11 +934,7 @@ Folder *FolderMan::addFolderFromFolderWizardResult(const AccountStatePtr &accoun
 
 QString FolderMan::suggestSyncFolder(const QUrl &server, const QString &displayName)
 {
-    QString folderName = tr("%1 - %2@%3").arg(Theme::instance()->appName(), displayName, server.host());
-    if (!Theme::instance()->multiAccount()) {
-        folderName = Theme::instance()->appName();
-    }
-    return FolderMan::instance()->findGoodPathForNewSyncFolder(QDir::homePath(), folderName);
+    return FolderMan::instance()->findGoodPathForNewSyncFolder(QDir::homePath(), Theme::instance()->appName());
 }
 
 bool FolderMan::prepareFolder(const QString &folder)
