@@ -17,28 +17,19 @@
 
 #include "account.h"
 #include "accountmanager.h"
-#include "accountstate.h"
-#include "common/syncjournalfilerecord.h"
 #include "commonstrings.h"
-#include "configfile.h"
-#include "elidedlabel.h"
 #include "folder.h"
 #include "folderman.h"
 #include "issueswidget.h"
-#include "logger.h"
+#include "libsync/configfile.h"
 #include "models/models.h"
-#include "openfilemanager.h"
 #include "protocolwidget.h"
 #include "syncengine.h"
 #include "syncfileitem.h"
-#include "syncresult.h"
 #include "theme.h"
 
-
 #include "ui_issueswidget.h"
 
-#include <climits>
-
 namespace {
 bool persistsUntilLocalDiscovery(const OCC::ProtocolItem &data)
 {
@@ -58,7 +49,7 @@ public:
     explicit SyncFileItemStatusSetSortFilterProxyModel(QObject *parent = nullptr)
         : Models::SignalledQSortFilterProxyModel(parent)
     {
-        resetFilter();
+        restoreFilter();
     }
 
     ~SyncFileItemStatusSetSortFilterProxyModel() override
@@ -70,10 +61,13 @@ public:
         return _filter;
     }
 
-    void setFilter(const StatusSet &newFilter)
+    void setFilter(const StatusSet &newFilter, bool save = true)
     {
         if (_filter != newFilter) {
             _filter = newFilter;
+            if (save) {
+                saveFilter();
+            }
             invalidateFilter();
             emit filterChanged();
         }
@@ -138,8 +132,50 @@ private:
         return defaultSet;
     }
 
+    void saveFilter()
+    {
+        QStringList checked;
+        for (uint8_t s = SyncFileItem::NoStatus; s < SyncFileItem::StatusCount; ++s) {
+            if (_filter[s]) {
+                checked.append(Utility::enumToString(static_cast<SyncFileItem::Status>(s)));
+            }
+        }
+        ConfigFile().setIssuesWidgetFilter(checked);
+    }
+
+    void restoreFilter()
+    {
+        StatusSet filter = {};
+        bool filterNeedsReset = true; // If there is no filter, the `true` value will cause a reset.
+        std::optional<QStringList> checked = ConfigFile().issuesWidgetFilter();
+
+        if (checked.has_value()) {
+            // There is a filter, but it can be empty (user unchecked all checkboxes), and in that case we do not want to reset the filter.
+            filterNeedsReset = false;
+
+            for (const QString &s : checked.value()) {
+                auto status = Utility::stringToEnum<SyncFileItem::Status>(s);
+                if (static_cast<int8_t>(status) == -1) {
+                    // The string value is not a valid enum value, so stop processing, and queue a reset.
+                    filterNeedsReset = true;
+                    break;
+                } else {
+                    filter[status] = true;
+                }
+            }
+        }
+
+        if (filterNeedsReset) {
+            // If there was no filter in the config file, or if one of the values is invalid, reset the filter.
+            resetFilter();
+        } else {
+            // There is a valid filter, so apply it. Also don't save it, because we just loaded it successfully
+            setFilter(filter, false);
+        }
+    }
+
 private:
-    StatusSet _filter;
+    StatusSet _filter = {};
 };
 
 /**
@@ -165,17 +201,11 @@ IssuesWidget::IssuesWidget(QWidget *parent)
             _model->addProtocolItem(ProtocolItem { folder, item });
         });
 
-    connect(ProgressDispatcher::instance(), &ProgressDispatcher::excluded, this, [this](Folder *f, const QString &file, CSYNC_EXCLUDE_TYPE reason) {
+    connect(ProgressDispatcher::instance(), &ProgressDispatcher::excluded, this, [this](Folder *f, const QString &file) {
         auto item = SyncFileItemPtr::create();
-        item->_status = SyncFileItem::FileIgnored;
+        item->_status = SyncFileItem::FilenameReserved;
         item->_file = file;
-        switch (reason) {
-        case CSYNC_FILE_EXCLUDE_RESERVED:
-            item->_errorString = tr("The file %1 was ignored as its name is reserved by %2").arg(file, Theme::instance()->appNameGUI());
-            break;
-        default:
-            Q_UNREACHABLE();
-        }
+        item->_errorString = tr("The file %1 was ignored as its name is reserved by %2").arg(file, Theme::instance()->appNameGUI());
         _model->addProtocolItem(ProtocolItem { f, item });
     });
 
@@ -183,7 +213,7 @@ IssuesWidget::IssuesWidget(QWidget *parent)
     _sortModel = new Models::SignalledQSortFilterProxyModel(this);
     connect(_sortModel, &Models::SignalledQSortFilterProxyModel::filterChanged, this, &IssuesWidget::filterDidChange);
     _sortModel->setSourceModel(_model);
-    _statusSortModel = new SyncFileItemStatusSetSortFilterProxyModel(this);
+    _statusSortModel = new SyncFileItemStatusSetSortFilterProxyModel(this); // Note: this will restore a previously set filter, if there was one.
     connect(_statusSortModel, &Models::SignalledQSortFilterProxyModel::filterChanged, this, &IssuesWidget::filterDidChange);
     _statusSortModel->setSourceModel(_sortModel);
     _statusSortModel->setSortRole(Qt::DisplayRole); // Sorting should be done based on the text in the column cells, but...
@@ -207,6 +237,7 @@ IssuesWidget::IssuesWidget(QWidget *parent)
     connect(_ui->_filterButton, &QAbstractButton::clicked, this, [this] {
         showFilterMenu(_ui->_filterButton);
     });
+    filterDidChange(); // Set the appropriate label.
 
     _ui->_tooManyIssuesWarning->hide();
     connect(_model, &ProtocolItemModel::rowsInserted, this, [this] {
@@ -273,7 +304,7 @@ void IssuesWidget::slotProgressInfo(Folder *folder, const ProgressInfo &progress
             if (item.folder() != folder) {
                 return false;
             }
-            if (item.direction() == SyncFileItem::None && item.status() != SyncFileItem::Excluded && item.status() != SyncFileItem::Conflict) {
+            if (item.direction() == SyncFileItem::None && item.status() == SyncFileItem::FilenameReserved) {
                 // TODO: don't clear syncErrors and excludes for now.
                 // make them either unique or remove them on the next sync?
                 return false;
@@ -300,10 +331,9 @@ void IssuesWidget::slotProgressInfo(Folder *folder, const ProgressInfo &progress
         // We keep track very well of pending conflicts.
         // Inform other components about them.
         QStringList conflicts;
-        for (const auto &data : _model->rawData()) {
-            if (data.folder() == folder
-                && data.status() == SyncFileItem::Conflict) {
-                conflicts.append(data.path());
+        for (const auto &rawData : _model->rawData()) {
+            if (rawData.folder() == folder && rawData.status() == SyncFileItem::Conflict) {
+                conflicts.append(rawData.path());
             }
         }
         emit ProgressDispatcher::instance()->folderConflicts(folder, conflicts);
@@ -324,11 +354,11 @@ void IssuesWidget::filterDidChange()
     // We have two filters here: the filter by status (which can have multiple items checked *off*...
     int filterCount = _statusSortModel->filterCount();
     // .. and the filter on the account name, which can only be 1 item checked:
-    if (!_sortModel->filterRegExp().isEmpty()) {
+    if (!_sortModel->filterRegularExpression().pattern().isEmpty()) {
         filterCount += 1;
     }
 
-    _ui->_filterButton->setText(CommonStrings::filterButtonText(filterCount));
+    _ui->_filterButton->setText(filterCount > 0 ? CommonStrings::filterButtonText(filterCount) : tr("Filter"));
 }
 
 void IssuesWidget::slotItemContextMenu()
@@ -352,46 +382,30 @@ std::function<void(void)> IssuesWidget::addStatusFilter(QMenu *menu)
     const auto initialFilter = _statusSortModel->filter();
 
     { // Add all errors under 1 action:
-        const std::vector<SyncFileItem::Status> ErrorStatusItems = {
-            SyncFileItem::Status::FatalError,
-            SyncFileItem::Status::NormalError,
-            SyncFileItem::Status::SoftError,
-            SyncFileItem::Status::DetailError,
-        };
-
-        auto action = menu->addAction(Utility::enumToDisplayName(SyncFileItem::NormalError), this, [this, ErrorStatusItems](bool checked) {
+        auto action = menu->addAction(Utility::enumToDisplayName(SyncFileItem::NormalError), this, [this](bool checked) {
             auto currentFilter = _statusSortModel->filter();
-            for (const auto &item : ErrorStatusItems) {
+            for (const auto &item : SyncFileItem::ErrorStatusItems) {
                 currentFilter[item] = checked;
             }
             _statusSortModel->setFilter(currentFilter);
         });
         action->setCheckable(true);
-        action->setChecked(initialFilter[ErrorStatusItems[0]]);
+        action->setChecked(initialFilter[SyncFileItem::ErrorStatusItems[0]]);
         statusFilterGroup->addAction(action);
     }
     menu->addSeparator();
 
-    // Add the other non-error items:
-    const std::vector<SyncFileItem::Status> OtherStatusItems = {
-        SyncFileItem::Status::Conflict,
-        SyncFileItem::Status::FileIgnored,
-        SyncFileItem::Status::Restoration,
-        SyncFileItem::Status::BlacklistedError,
-        SyncFileItem::Status::Excluded,
-        SyncFileItem::Status::Message
-    };
-    // list of OtherStatusItems with the localised name
+    // list of OtherDisplayableStatusItems with the localised name
     std::vector<std::pair<QString, SyncFileItem::Status>> otherStatusItems;
-    otherStatusItems.reserve(OtherStatusItems.size());
-    for (const auto &item : OtherStatusItems) {
+    otherStatusItems.reserve(SyncFileItem::OtherDisplayableStatusItems.size());
+    for (const auto &item : SyncFileItem::OtherDisplayableStatusItems) {
         otherStatusItems.emplace_back(Utility::enumToDisplayName(item), item);
     }
     std::sort(otherStatusItems.begin(), otherStatusItems.end(), [](const auto &a, const auto &b) {
         return a.first < b.first;
     });
     for (const auto &item : otherStatusItems) {
-        auto action = menu->addAction(item.first, this, [this, item](bool checked) {
+        auto action = menu->addAction(item.first, menu, [this, item](bool checked) {
             auto currentFilter = _statusSortModel->filter();
             currentFilter[item.second] = checked;
             _statusSortModel->setFilter(currentFilter);
