@@ -22,6 +22,7 @@ class ExpandingHeaderView : public QHeaderView
     Q_OBJECT
 public:
     ExpandingHeaderView(const QString &objectName, QWidget *parent = nullptr);
+    ~ExpandingHeaderView();
 
     int expandingColumn() const;
     void setExpandingColumn(int newExpandingColumn);
@@ -29,11 +30,16 @@ public:
     void resizeColumns(bool reset = false);
     void addResetActionToMenu(QMenu *menu);
 
+    bool resizeToContent() const;
+    void setResizeToContent(bool newResizeToContent);
+
 protected:
     void resizeEvent(QResizeEvent *event) override;
 
 
 private:
+    bool _requiresReset = false;
+    bool _resizeToContent = false;
     int _expandingColumn = 0;
 };
 
