@@ -13,6 +13,7 @@
  */
 
 #include "settingsdialog_mac.h"
+#include "common/utility.h"
 
 #import <AppKit/AppKit.h>
 #include <QDebug>
@@ -31,7 +32,10 @@ void setActivationPolicy(ActivationPolicy policy)
         mode = NSApplicationActivationPolicyProhibited;
         break;
     }
-    if (![NSApp setActivationPolicy:mode]) {
-        qWarning() << "setActivationPolicy" << static_cast<int>(policy) << "failed";
+
+    if (mode != NSApp.activationPolicy) {
+        if (![NSApp setActivationPolicy:mode]) {
+            qWarning() << "setActivationPolicy" << static_cast<int>(policy) << "failed";
+        }
     }
 }
