@@ -13,38 +13,34 @@
  */
 
 #include "shareusergroupwidget.h"
-#include "ui_shareusergroupwidget.h"
-#include "ui_shareuserline.h"
 #include "account.h"
-#include "folderman.h"
-#include "folder.h"
 #include "accountmanager.h"
-#include "theme.h"
-#include "configfile.h"
 #include "capabilities.h"
+#include "configfile.h"
 #include "guiutility.h"
-#include "thumbnailjob.h"
 #include "sharee.h"
 #include "sharemanager.h"
-#include "guiutility.h"
+#include "thumbnailjob.h"
+#include "ui_shareusergroupwidget.h"
+#include "ui_shareuserline.h"
+
+#include "resources/resources.h"
 
 #include "QProgressIndicator.h"
-#include <QBuffer>
-#include <QFileIconProvider>
-#include <QClipboard>
-#include <QFileInfo>
+
 #include <QAbstractProxyModel>
-#include <QCompleter>
-#include <qscrollarea.h>
-#include <qlayout.h>
-#include <QPropertyAnimation>
-#include <QMenu>
 #include <QAction>
-#include <QDesktopServices>
-#include <QMessageBox>
-#include <QCryptographicHash>
+#include <QBuffer>
+#include <QClipboard>
 #include <QColor>
+#include <QCompleter>
+#include <QCryptographicHash>
+#include <QDesktopServices>
+#include <QFileInfo>
+#include <QMenu>
+#include <QMessageBox>
 #include <QPainter>
+#include <QPropertyAnimation>
 
 namespace OCC {
 
@@ -64,7 +60,7 @@ ShareUserGroupWidget::ShareUserGroupWidget(AccountPtr account,
     , _disableCompleterActivated(false)
 {
     setAttribute(Qt::WA_DeleteOnClose);
-    setObjectName("SharingDialogUG"); // required as group for saveGeometry call
+    setObjectName(QStringLiteral("SharingDialogUG")); // required as group for saveGeometry call
 
     _ui->setupUi(this);
 
@@ -73,7 +69,7 @@ ShareUserGroupWidget::ShareUserGroupWidget(AccountPtr account,
 
     _completer = new QCompleter(this);
     _completerModel = new ShareeModel(_account,
-        _isFile ? QLatin1String("file") : QLatin1String("folder"),
+        _isFile ? QStringLiteral("file") : QStringLiteral("folder"),
         _completer);
     connect(_completerModel, &ShareeModel::shareesReady, this, &ShareUserGroupWidget::slotShareesReady);
     connect(_completerModel, &ShareeModel::displayErrorMessage, this, &ShareUserGroupWidget::displayError);
@@ -90,6 +86,10 @@ ShareUserGroupWidget::ShareUserGroupWidget(AccountPtr account,
     connect(_ui->shareeLineEdit, &QLineEdit::returnPressed, this, &ShareUserGroupWidget::slotLineEditReturn);
     connect(_ui->privateLinkText, &QLabel::linkActivated, this, &ShareUserGroupWidget::slotPrivateLinkShare);
 
+    if (account->capabilities().privateLinkPropertyAvailable()) {
+        _ui->privateLinkText->setText(tr("You can direct people to this shared file or folder %1 by giving them a private link")
+                                          .arg(QStringLiteral("<a href=\"%1\"><span style=\"text-decoration: underline\">").arg(_privateLinkUrl)));
+    }
     // By making the next two QueuedConnections we can override
     // the strings the completer sets on the line edit.
     connect(_completer, qOverload<const QModelIndex &>(&QCompleter::activated), this, &ShareUserGroupWidget::slotCompleterActivated,
@@ -161,7 +161,7 @@ void ShareUserGroupWidget::searchForSharees()
     ShareeModel::ShareeSet blacklist;
 
     // Add the current user to _sharees since we can't share with ourself
-    QSharedPointer<Sharee> currentUser(new Sharee(_account->credentials()->user(), "", Sharee::Type::User));
+    QSharedPointer<Sharee> currentUser(new Sharee(_account->credentials()->user(), QString(), Sharee::Type::User));
     blacklist << currentUser;
 
     const auto &shareUserLines = _ui->scrollArea->findChildren<ShareUserLine *>();
@@ -183,7 +183,7 @@ void ShareUserGroupWidget::slotSharesFetched(const QList<QSharedPointer<Share>>
 
     auto newViewPort = new QWidget(scrollArea);
     auto layout = new QVBoxLayout(newViewPort);
-    layout->setMargin(0);
+    layout->setContentsMargins({});
     layout->setSpacing(0);
 
     QSize minimumSize = newViewPort->sizeHint();
@@ -291,20 +291,8 @@ void ShareUserGroupWidget::slotCompleterActivated(const QModelIndex &index)
      * https://github.com/owncloud/client/issues/4996
      */
     const SharePermissions defaultPermissions = static_cast<SharePermissions>(_account->capabilities().defaultPermissions());
-    if (sharee->type() == Sharee::Federated
-        && _account->serverVersionInt() < Account::makeServerVersion(9, 1, 0)) {
-        SharePermissions permissions = SharePermissionRead | SharePermissionUpdate;
-        if (!_isFile) {
-            permissions |= SharePermissionCreate | SharePermissionDelete;
-        }
-        permissions &= defaultPermissions;
-        _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
-            sharee->shareWith(), permissions);
-    } else {
-        _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
-            sharee->shareWith(), _maxSharingPermissions & defaultPermissions);
-    }
-
+    _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
+        sharee->shareWith(), _maxSharingPermissions & defaultPermissions);
     _ui->shareeLineEdit->setEnabled(false);
     _ui->shareeLineEdit->setText(QString());
 }
@@ -331,7 +319,7 @@ void ShareUserGroupWidget::displayError(int code, const QString &message)
 
 void ShareUserGroupWidget::slotPrivateLinkOpenBrowser()
 {
-    Utility::openBrowser(_privateLinkUrl, this);
+    Utility::openBrowser(QUrl(_privateLinkUrl), this);
 }
 
 void ShareUserGroupWidget::slotPrivateLinkCopy()
@@ -382,7 +370,7 @@ ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
     }
     _ui->permissionToolButton->setMenu(menu);
     _ui->permissionToolButton->setPopupMode(QToolButton::InstantPopup);
-    _ui->permissionToolButton->setIcon(Utility::getCoreIcon(QStringLiteral("more")));
+    _ui->permissionToolButton->setIcon(Resources::getCoreIcon(QStringLiteral("more")));
 
     // If there's only a single entry in the detailed permission menu, hide it
     if (menu->actions().size() == 1) {
@@ -402,22 +390,10 @@ ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
     connect(_ui->permissionShare, &QAbstractButton::clicked, this, &ShareUserLine::slotPermissionsChanged);
     connect(_ui->permissionsEdit, &QAbstractButton::clicked, this, &ShareUserLine::slotEditPermissionsChanged);
 
-    /*
-     * We don't show permssion share for federated shares with server <9.1
-     * https://github.com/owncloud/core/issues/22122#issuecomment-185637344
-     * https://github.com/owncloud/client/issues/4996
-     */
-    if (share->getShareType() == Share::TypeRemote
-        && share->account()->serverVersionInt() < Account::makeServerVersion(9, 1, 0)) {
-        _ui->permissionShare->setVisible(false);
-        _ui->permissionToolButton->setVisible(false);
-    }
-
     connect(share.data(), &Share::permissionsSet, this, &ShareUserLine::slotPermissionsSet);
     connect(share.data(), &Share::shareDeleted, this, &ShareUserLine::slotShareDeleted);
 
-    _ui->deleteShareButton->setIcon(QIcon::fromTheme(QStringLiteral("user-trash"),
-        Utility::getCoreIcon(QStringLiteral("delete"))));
+    _ui->deleteShareButton->setIcon(QIcon::fromTheme(QStringLiteral("user-trash"), Resources::getCoreIcon(QStringLiteral("delete"))));
 
     if (!share->account()->capabilities().shareResharing()) {
         _ui->permissionShare->hide();
@@ -445,15 +421,16 @@ void ShareUserLine::loadAvatar()
     double hue = static_cast<quint8>(hash[0]) / 255.;
 
     // See core/js/placeholder.js for details on colors and styling
-    const QColor bg = QColor::fromHslF(hue, 0.7, 0.68);
-    const QString style = QString(R"(* {
+    const QColor bg = QColor::fromHslF(hue, 0.7f, 0.68f);
+    const QString style = QStringLiteral(R"(* {
         color: #fff;
         background-color: %1;
         border-radius: %2px;
         text-align: center;
         line-height: %2px;
         font-size: %2px;
-    })").arg(bg.name(), QString::number(avatarSize / 2));
+    })")
+                              .arg(bg.name(), QString::number(avatarSize / 2));
     _ui->avatar->setStyleSheet(style);
 
     // The avatar label is the first character of the user name.
@@ -484,7 +461,7 @@ void ShareUserLine::slotAvatarLoaded(const QPixmap &avatar)
     _ui->avatar->setPixmap(_avatar);
 
     // Remove the stylesheet for the fallback avatar
-    _ui->avatar->setStyleSheet("");
+    _ui->avatar->setStyleSheet(QString());
 }
 
 void ShareUserLine::on_deleteShareButton_clicked()
