@@ -136,16 +136,20 @@ bool AbstractNetworkJob::needsRetry() const
     }
 
     if (auto reply = this->reply()) {
+        // we had an unsupported redirect
         if (!reply->attribute(QNetworkRequest::RedirectionTargetAttribute).isNull()) {
             return true;
         }
-        if (reply->error() != QNetworkReply::NoError) {
-            if (reply->error() == QNetworkReply::AuthenticationRequiredError) {
+        switch (reply->error()) {
+        case QNetworkReply::AuthenticationRequiredError:
+            return true;
+        case QNetworkReply::ContentReSendError:
+            if (_reply->attribute(QNetworkRequest::Http2WasUsedAttribute).toBool()) {
                 return true;
             }
-        }
-        if (_reply->error() == QNetworkReply::ContentReSendError && _reply->attribute(QNetworkRequest::Http2WasUsedAttribute).toBool()) {
-            return true;
+            [[fallthrough]];
+        default:
+            break;
         }
     }
     return false;
@@ -157,6 +161,10 @@ void AbstractNetworkJob::sendRequest(const QByteArray &verb,
     _verb = verb;
 
     _request = req;
+
+    // allow to transfer files with a big compression ratio
+    _request.setDecompressedSafetyCheckThreshold(-1);
+
     _request.setAttribute(QNetworkRequest::CacheSaveControlAttribute, _storeInCache);
 
     if (_cacheLoadControl.has_value()) {
@@ -246,7 +254,7 @@ QByteArray AbstractNetworkJob::responseTimestamp() const
 
 QDateTime OCC::AbstractNetworkJob::responseQTimeStamp() const
 {
-    return QDateTime::fromString(QString::fromUtf8(responseTimestamp()), Qt::RFC2822Date);
+    return Utility::parseRFC1123Date(QString::fromUtf8(responseTimestamp()));
 }
 
 QByteArray AbstractNetworkJob::requestId()
@@ -291,7 +299,7 @@ QString AbstractNetworkJob::errorStringParsingBody(QByteArray *body)
 AbstractNetworkJob::~AbstractNetworkJob()
 {
     if (!_finished && !_aborted && !_timedout) {
-        qCCritical(lcNetworkJob) << "Deleting running job" << this << parent();
+        qCCritical(lcNetworkJob) << "Deleting running job" << this;
     }
     delete _reply;
     _reply = nullptr;
