@@ -12,24 +12,22 @@
  * for more details.
  */
 
-#include "config.h"
-
-#include "common/utility.h"
 #include "common/asserts.h"
+#include "common/utility.h"
+#include "common/version.h"
+#ifdef Q_OS_WIN
+#include "common/utility_win.h"
+#endif
 #include "configfile.h"
 #include "logger.h"
 #include "theme.h"
-#include "version.h"
 
 #include "creds/abstractcredentials.h"
 
 #include "csync_exclude.h"
 
-#ifndef TOKEN_AUTH_ONLY
 #include <QWidget>
 #include <QHeaderView>
-#endif
-
 #include <QCoreApplication>
 #include <QDir>
 #include <QFile>
@@ -40,7 +38,8 @@
 #include <QOperatingSystemVersion>
 #include <QStandardPaths>
 
-#define DEFAULT_MAX_LOG_LINES 20000
+#include <chrono>
+using namespace std::chrono_literals;
 
 namespace OCC {
 
@@ -57,8 +56,10 @@ const QString notificationRefreshIntervalC() { return QStringLiteral("notificati
 const QString monoIconsC() { return QStringLiteral("monoIcons"); }
 const QString promptDeleteC() { return QStringLiteral("promptDeleteAllFiles"); }
 const QString crashReporterC() { return QStringLiteral("crashReporter"); }
-const QString optionalDesktopNoficationsC() { return QStringLiteral("optionalDesktopNotifications"); }
-const QString showInExplorerNavigationPaneC() { return QStringLiteral("showInExplorerNavigationPane"); }
+const QString optionalDesktopNoficationsC()
+{
+    return QStringLiteral("optionalDesktopNotifications");
+}
 const QString skipUpdateCheckC() { return QStringLiteral("skipUpdateCheck"); }
 const QString updateCheckIntervalC() { return QStringLiteral("updateCheckInterval"); }
 const QString updateChannelC() { return QStringLiteral("updateChannel"); }
@@ -70,15 +71,20 @@ const QString minChunkSizeC() { return QStringLiteral("minChunkSize"); }
 const QString maxChunkSizeC() { return QStringLiteral("maxChunkSize"); }
 const QString targetChunkUploadDurationC() { return QStringLiteral("targetChunkUploadDuration"); }
 const QString automaticLogDirC() { return QStringLiteral("logToTemporaryLogDir"); }
-const QString deleteOldLogsAfterHoursC() { return QStringLiteral("temporaryLogDirDeleteOldLogsAfterHours"); }
+const QString numberOfLogsToKeepC() { return QStringLiteral("numberOfLogsToKeep"); }
 const QString showExperimentalOptionsC() { return QStringLiteral("showExperimentalOptions"); }
+
+// The key `clientVersion` stores the version *with* build number of the config file. It is named
+// this way, because before 5.0, only the version *without* build number was stored.
 const QString clientVersionC() { return QStringLiteral("clientVersion"); }
 
 const QString proxyHostC() { return QStringLiteral("Proxy/host"); }
 const QString proxyTypeC() { return QStringLiteral("Proxy/type"); }
 const QString proxyPortC() { return QStringLiteral("Proxy/port"); }
-const QString proxyUserC() { return QStringLiteral("Proxy/user"); }
-const QString proxyPassC() { return QStringLiteral("Proxy/pass"); }
+const QString proxyUserC()
+{
+    return QStringLiteral("Proxy/user");
+}
 const QString proxyNeedsAuthC() { return QStringLiteral("Proxy/needsAuth"); }
 
 const QString useUploadLimitC() { return QStringLiteral("BWLimit/useUploadLimit"); }
@@ -90,8 +96,20 @@ const QString newBigFolderSizeLimitC() { return QStringLiteral("newBigFolderSize
 const QString useNewBigFolderSizeLimitC() { return QStringLiteral("useNewBigFolderSizeLimit"); }
 const QString confirmExternalStorageC() { return QStringLiteral("confirmExternalStorage"); }
 const QString moveToTrashC() { return QStringLiteral("moveToTrash"); }
+
+const QString issuesWidgetFilterC()
+{
+    return QStringLiteral("issuesWidgetFilter");
 }
 
+QString excludeFileNameC()
+{
+    static_assert(!std::string_view(EXCLUDE_FILE_NAME).empty());
+    return QStringLiteral(EXCLUDE_FILE_NAME);
+}
+
+} // anonymous namespace
+
 QString ConfigFile::_confDir = QString();
 const std::chrono::seconds DefaultRemotePollInterval { 30 };
 
@@ -104,18 +122,6 @@ static chrono::milliseconds millisecondsValue(const QSettings &setting, const QS
 ConfigFile::ConfigFile()
 {
     QSettings::setDefaultFormat(QSettings::IniFormat);
-
-    const QString config = configFile();
-
-    QSettings settings(config, QSettings::IniFormat);
-    settings.beginGroup(defaultConnection());
-
-    // run init only once
-    static bool init = [this]() {
-        setLogHttp(logHttp());
-        return false;
-    }();
-    Q_UNUSED(init);
 }
 
 bool ConfigFile::setConfDir(const QString &value)
@@ -140,118 +146,116 @@ bool ConfigFile::setConfDir(const QString &value)
 
 bool ConfigFile::optionalDesktopNotifications() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(optionalDesktopNoficationsC(), true).toBool();
 }
 
-bool ConfigFile::showInExplorerNavigationPane() const
+std::optional<QStringList> ConfigFile::issuesWidgetFilter() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
-    return settings.value(showInExplorerNavigationPaneC(), QOperatingSystemVersion::current() >= QOperatingSystemVersion::Windows10).toBool();
+    auto settings = makeQSettings();
+    if (settings.contains(issuesWidgetFilterC())) {
+        return settings.value(issuesWidgetFilterC()).toStringList();
+    }
+
+    return {};
 }
 
-void ConfigFile::setShowInExplorerNavigationPane(bool show)
+void ConfigFile::setIssuesWidgetFilter(const QStringList &checked)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
-    settings.setValue(showInExplorerNavigationPaneC(), show);
+    auto settings = makeQSettings();
+    settings.setValue(issuesWidgetFilterC(), checked);
     settings.sync();
 }
 
-int ConfigFile::timeout() const
+std::chrono::seconds ConfigFile::timeout() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
-    return settings.value(timeoutC(), 300).toInt(); // default to 5 min
+    auto settings = makeQSettings();
+    const auto val = settings.value(timeoutC()).toInt(); // default to 5 min
+    return val ? std::chrono::seconds(val) : 5min;
 }
 
 qint64 ConfigFile::chunkSize() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(chunkSizeC(), 10 * 1000 * 1000).toLongLong(); // default to 10 MB
 }
 
 qint64 ConfigFile::maxChunkSize() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(maxChunkSizeC(), 100 * 1000 * 1000).toLongLong(); // default to 100 MB
 }
 
 qint64 ConfigFile::minChunkSize() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(minChunkSizeC(), 1000 * 1000).toLongLong(); // default to 1 MB
 }
 
 chrono::milliseconds ConfigFile::targetChunkUploadDuration() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return millisecondsValue(settings, targetChunkUploadDurationC(), chrono::minutes(1));
 }
 
 void ConfigFile::setOptionalDesktopNotifications(bool show)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(optionalDesktopNoficationsC(), show);
     settings.sync();
 }
 
 void ConfigFile::saveGeometry(QWidget *w)
 {
-#ifndef TOKEN_AUTH_ONLY
     OC_ASSERT(!w->objectName().isNull());
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(w->objectName());
     settings.setValue(geometryC(), w->saveGeometry());
     settings.sync();
-#endif
 }
 
 void ConfigFile::restoreGeometry(QWidget *w)
 {
-#ifndef TOKEN_AUTH_ONLY
     w->restoreGeometry(getValue(geometryC(), w->objectName()).toByteArray());
-#endif
 }
 
 void ConfigFile::saveGeometryHeader(QHeaderView *header)
 {
-#ifndef TOKEN_AUTH_ONLY
     if (!header)
         return;
     OC_ASSERT(!header->objectName().isEmpty());
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(header->objectName());
     settings.setValue(geometryC(), header->saveState());
     settings.sync();
-#endif
 }
 
-void ConfigFile::restoreGeometryHeader(QHeaderView *header)
+bool ConfigFile::restoreGeometryHeader(QHeaderView *header)
 {
-#ifndef TOKEN_AUTH_ONLY
-    if (!header)
-        return;
-    OC_ASSERT(!header->objectName().isNull());
+    Q_ASSERT(header && !header->objectName().isNull());
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(header->objectName());
-    header->restoreState(settings.value(geometryC()).toByteArray());
-#endif
+    if (settings.contains(geometryC())) {
+        header->restoreState(settings.value(geometryC()).toByteArray());
+        return true;
+    }
+    return false;
 }
 
 QVariant ConfigFile::getPolicySetting(const QString &setting, const QVariant &defaultValue) const
 {
     if (Utility::isWindows()) {
         // check for policies first and return immediately if a value is found.
-        QSettings userPolicy(QStringLiteral("HKEY_CURRENT_USER\\Software\\Policies\\%1\\%2")
-                                 .arg(QStringLiteral(APPLICATION_VENDOR), Theme::instance()->appNameGUI()),
+        QSettings userPolicy(QStringLiteral("HKEY_CURRENT_USER\\Software\\Policies\\%1\\%2").arg(Theme::instance()->vendor(), Theme::instance()->appNameGUI()),
             QSettings::NativeFormat);
         if (userPolicy.contains(setting)) {
             return userPolicy.value(setting);
         }
 
-        QSettings machinePolicy(QStringLiteral("HKEY_LOCAL_MACHINE\\Software\\Policies\\%1\\%2")
-                                    .arg(QStringLiteral(APPLICATION_VENDOR), Theme::instance()->appNameGUI()),
+        QSettings machinePolicy(
+            QStringLiteral("HKEY_LOCAL_MACHINE\\Software\\Policies\\%1\\%2").arg(Theme::instance()->vendor(), Theme::instance()->appNameGUI()),
             QSettings::NativeFormat);
         if (machinePolicy.contains(setting)) {
             return machinePolicy.value(setting);
@@ -274,8 +278,6 @@ QString ConfigFile::configPath()
     return dir;
 }
 
-static const QLatin1String exclFile("sync-exclude.lst");
-
 QString ConfigFile::excludeFile(Scope scope) const
 {
 #ifdef Q_OS_WIN
@@ -288,13 +290,13 @@ QString ConfigFile::excludeFile(Scope scope) const
 
     switch (scope) {
     case UserScope:
-        fi.setFile(configPath(), exclFile);
+        fi.setFile(configPath(), excludeFileNameC());
 
         if (!fi.isReadable()) {
             fi.setFile(configPath(), QStringLiteral("exclude.lst"));
         }
         if (!fi.isReadable()) {
-            fi.setFile(configPath(), exclFile);
+            fi.setFile(configPath(), excludeFileNameC());
         }
         return fi.absoluteFilePath();
     case SystemScope:
@@ -302,43 +304,37 @@ QString ConfigFile::excludeFile(Scope scope) const
     }
 
     OC_ASSERT(false);
-    return QString();
+    return {};
 }
 
 QString ConfigFile::excludeFileFromSystem()
 {
     QFileInfo fi;
 #ifdef Q_OS_WIN
-    fi.setFile(QCoreApplication::applicationDirPath(), exclFile);
+    fi.setFile(QCoreApplication::applicationDirPath(), excludeFileNameC());
 #endif
 #ifdef Q_OS_UNIX
-    fi.setFile(QStringLiteral(SYSCONFDIR "/%1").arg(Theme::instance()->appName()), exclFile);
+    fi.setFile(QStringLiteral(SYSCONFDIR "/%1").arg(Theme::instance()->appName()), excludeFileNameC());
     if (!fi.exists()) {
         // Prefer to return the preferred path! Only use the fallback location
         // if the other path does not exist and the fallback is valid.
-        QFileInfo nextToBinary(QCoreApplication::applicationDirPath(), exclFile);
+        QFileInfo nextToBinary(QCoreApplication::applicationDirPath(), excludeFileNameC());
         if (nextToBinary.exists()) {
             fi = nextToBinary;
         } else {
-            // For AppImage, the file might reside under a temporary mount path
-            QDir d(QCoreApplication::applicationDirPath()); // supposed to be /tmp/mount.xyz/usr/bin
-            d.cdUp(); // go out of bin
-            d.cdUp(); // go out of usr
-            if (!d.isRoot()) { // it is really a mountpoint
-                if (d.cd(QStringLiteral("etc")) && d.cd(Theme::instance()->appName())) {
-                    QFileInfo inMountDir(d, exclFile);
-                    if (inMountDir.exists()) {
-                        fi = inMountDir;
-                    }
-                };
+            // use from install tree (e.g., AppImage, local dev installation)
+            // for example, if the binary is in .../AppDir/usr/bin/<binary>, the exclude file will be in .../AppDir/usr/etc/<appname>/
+            QFileInfo relativeToBinary(
+                QStringLiteral("%1/../etc/%2/%3").arg(QCoreApplication::applicationDirPath(), Theme::instance()->appName(), excludeFileNameC()));
+            if (relativeToBinary.exists()) {
+                fi = relativeToBinary;
             }
         }
     }
 #endif
 #ifdef Q_OS_MAC
     // exec path is inside the bundle
-    fi.setFile(QCoreApplication::applicationDirPath(),
-        QLatin1String("../Resources/") + exclFile);
+    fi.setFile(QCoreApplication::applicationDirPath(), QLatin1String("../Resources/") + excludeFileNameC());
 #endif
 
     return fi.absoluteFilePath();
@@ -347,7 +343,7 @@ QString ConfigFile::excludeFileFromSystem()
 QString ConfigFile::backup() const
 {
     QString baseFile = configFile();
-    auto versionString = clientVersionString();
+    auto versionString = clientVersionWithBuildNumberString();
     if (!versionString.isEmpty())
         versionString.prepend(QLatin1Char('_'));
     const QString backupFile =
@@ -369,6 +365,11 @@ QString ConfigFile::configFile()
     return configPath() + Theme::instance()->configFileName();
 }
 
+QSettings ConfigFile::makeQSettings()
+{
+    return { configFile(), QSettings::IniFormat };
+}
+
 bool ConfigFile::exists()
 {
     return QFileInfo::exists(configFile());
@@ -382,7 +383,7 @@ QString ConfigFile::defaultConnection() const
 void ConfigFile::storeData(const QString &group, const QString &key, const QVariant &value)
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.beginGroup(con);
     settings.setValue(key, value);
@@ -392,7 +393,7 @@ void ConfigFile::storeData(const QString &group, const QString &key, const QVari
 void ConfigFile::removeData(const QString &group, const QString &key)
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.beginGroup(con);
     settings.remove(key);
@@ -401,7 +402,7 @@ void ConfigFile::removeData(const QString &group, const QString &key)
 bool ConfigFile::dataExists(const QString &group, const QString &key) const
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.beginGroup(con);
     return settings.contains(key);
@@ -413,7 +414,7 @@ chrono::milliseconds ConfigFile::remotePollInterval(std::chrono::seconds default
     if (connection.isEmpty())
         con = defaultConnection();
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     auto defaultPollInterval { DefaultRemotePollInterval };
@@ -444,7 +445,7 @@ void ConfigFile::setRemotePollInterval(chrono::milliseconds interval, const QStr
         qCWarning(lcConfigFile) << "Remote Poll interval of " << interval.count() << " is below five seconds.";
         return;
     }
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
     settings.setValue(remotePollIntervalC(), qlonglong(interval.count()));
     settings.sync();
@@ -457,7 +458,7 @@ chrono::milliseconds ConfigFile::forceSyncInterval(std::chrono::seconds remoteFr
     QString con(connection);
     if (connection.isEmpty())
         con = defaultConnection();
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     auto defaultInterval = chrono::hours(2);
@@ -471,9 +472,9 @@ chrono::milliseconds ConfigFile::forceSyncInterval(std::chrono::seconds remoteFr
 
 chrono::milliseconds OCC::ConfigFile::fullLocalDiscoveryInterval() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(defaultConnection());
-    return millisecondsValue(settings, fullLocalDiscoveryIntervalC(), chrono::hours(1));
+    return millisecondsValue(settings, fullLocalDiscoveryIntervalC(), 1h);
 }
 
 chrono::milliseconds ConfigFile::notificationRefreshInterval(const QString &connection) const
@@ -481,7 +482,7 @@ chrono::milliseconds ConfigFile::notificationRefreshInterval(const QString &conn
     QString con(connection);
     if (connection.isEmpty())
         con = defaultConnection();
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     auto defaultInterval = chrono::minutes(5);
@@ -498,7 +499,7 @@ chrono::milliseconds ConfigFile::updateCheckInterval(const QString &connection)
     QString con(connection);
     if (connection.isEmpty())
         con = defaultConnection();
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     auto defaultInterval = chrono::hours(10);
@@ -531,7 +532,7 @@ void ConfigFile::setSkipUpdateCheck(bool skip, const QString &connection)
     if (connection.isEmpty())
         con = defaultConnection();
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.beginGroup(con);
 
     settings.setValue(skipUpdateCheckC(), QVariant(skip));
@@ -541,7 +542,7 @@ void ConfigFile::setSkipUpdateCheck(bool skip, const QString &connection)
 QString ConfigFile::updateChannel() const
 {
     QString defaultUpdateChannel = QStringLiteral("stable");
-    const QString suffix = MIRALL_VERSION_SUFFIX();
+    const QString suffix = OCC::Version::suffix();
     if (suffix.startsWith(QLatin1String("daily"))
         || suffix.startsWith(QLatin1String("nightly"))
         || suffix.startsWith(QLatin1String("alpha"))
@@ -550,35 +551,31 @@ QString ConfigFile::updateChannel() const
         defaultUpdateChannel = QStringLiteral("beta");
     }
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(updateChannelC(), defaultUpdateChannel).toString();
 }
 
 void ConfigFile::setUpdateChannel(const QString &channel)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(updateChannelC(), channel);
 }
 
 QString ConfigFile::uiLanguage() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(uiLanguageC(), QString()).toString();
 }
 
 void ConfigFile::setUiLanguage(const QString &uiLanguage)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(uiLanguageC(), uiLanguage);
 }
 
-void ConfigFile::setProxyType(int proxyType,
-    const QString &host,
-    int port, bool needsAuth,
-    const QString &user,
-    const QString &pass)
+void ConfigFile::setProxyType(QNetworkProxy::ProxyType proxyType, const QString &host, int port, bool needsAuth, const QString &user)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.setValue(proxyTypeC(), proxyType);
 
@@ -587,7 +584,6 @@ void ConfigFile::setProxyType(int proxyType,
         settings.setValue(proxyPortC(), port);
         settings.setValue(proxyNeedsAuthC(), needsAuth);
         settings.setValue(proxyUserC(), user);
-        settings.setValue(proxyPassC(), pass.toUtf8().toBase64());
     }
     settings.sync();
 }
@@ -597,7 +593,7 @@ QVariant ConfigFile::getValue(const QString &param, const QString &group,
 {
     QVariant systemSetting;
     if (Utility::isMac()) {
-        QSettings systemSettings(QStringLiteral("/Library/Preferences/" APPLICATION_REV_DOMAIN ".plist"), QSettings::NativeFormat);
+        QSettings systemSettings(QStringLiteral("/Library/Preferences/%1.plist").arg(Theme::instance()->orgDomainName()), QSettings::NativeFormat);
         if (!group.isEmpty()) {
             systemSettings.beginGroup(group);
         }
@@ -609,16 +605,15 @@ QVariant ConfigFile::getValue(const QString &param, const QString &group,
         }
         systemSetting = systemSettings.value(param, defaultValue);
     } else { // Windows
-        QSettings systemSettings(QStringLiteral("HKEY_LOCAL_MACHINE\\Software\\" APPLICATION_VENDOR "\\%1")
-                                     .arg(Theme::instance()->appNameGUI()),
-            QSettings::NativeFormat);
+        QSettings systemSettings(
+            QStringLiteral("HKEY_LOCAL_MACHINE\\Software\\%1\\%2").arg(Theme::instance()->vendor(), Theme::instance()->appNameGUI()), QSettings::NativeFormat);
         if (!group.isEmpty()) {
             systemSettings.beginGroup(group);
         }
         systemSetting = systemSettings.value(param, defaultValue);
     }
 
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     if (!group.isEmpty())
         settings.beginGroup(group);
 
@@ -627,7 +622,7 @@ QVariant ConfigFile::getValue(const QString &param, const QString &group,
 
 void ConfigFile::setValue(const QString &key, const QVariant &value)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
 
     settings.setValue(key, value);
 }
@@ -660,12 +655,6 @@ QString ConfigFile::proxyUser() const
     return getValue(proxyUserC()).toString();
 }
 
-QString ConfigFile::proxyPassword() const
-{
-    QByteArray pass = getValue(proxyPassC()).toByteArray();
-    return QString::fromUtf8(QByteArray::fromBase64(pass));
-}
-
 int ConfigFile::useUploadLimit() const
 {
     return getValue(useUploadLimitC(), QString(), 0).toInt();
@@ -742,85 +731,81 @@ void ConfigFile::setMoveToTrash(bool isChecked)
 
 bool ConfigFile::promptDeleteFiles() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(promptDeleteC(), true).toBool();
 }
 
 void ConfigFile::setPromptDeleteFiles(bool promptDeleteFiles)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(promptDeleteC(), promptDeleteFiles);
 }
 
 bool ConfigFile::monoIcons() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     bool monoDefault = false; // On Mac we want bw by default
 #ifdef Q_OS_MAC
     // OEM themes are not obliged to ship mono icons
-    monoDefault = (0 == (strcmp("ownCloud", APPLICATION_NAME)));
+    monoDefault = Theme::instance()->appNameGUI() == QStringLiteral("ownCloud");
 #endif
     return settings.value(monoIconsC(), monoDefault).toBool();
 }
 
 void ConfigFile::setMonoIcons(bool useMonoIcons)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(monoIconsC(), useMonoIcons);
 }
 
 bool ConfigFile::crashReporter() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(crashReporterC(), true).toBool();
 }
 
 void ConfigFile::setCrashReporter(bool enabled)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(crashReporterC(), enabled);
 }
 
 bool ConfigFile::automaticLogDir() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(automaticLogDirC(), false).toBool();
 }
 
 void ConfigFile::setAutomaticLogDir(bool enabled)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(automaticLogDirC(), enabled);
 }
 
-Optional<chrono::hours> ConfigFile::automaticDeleteOldLogsAge() const
+int ConfigFile::automaticDeleteOldLogs() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
-    auto value = settings.value(deleteOldLogsAfterHoursC());
-    if (!value.isValid())
-        return chrono::hours(4);
-    auto hours = value.toInt();
-    if (hours <= 0)
-        return {};
-    return chrono::hours(hours);
+    auto settings = makeQSettings();
+    return settings.value(numberOfLogsToKeepC()).toInt();
 }
 
-void ConfigFile::setAutomaticDeleteOldLogsAge(Optional<chrono::hours> expireTime)
+void ConfigFile::setAutomaticDeleteOldLogs(int number)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
-    if (!expireTime) {
-        settings.setValue(deleteOldLogsAfterHoursC(), -1);
-    } else {
-        settings.setValue(deleteOldLogsAfterHoursC(), QVariant::fromValue(expireTime->count()));
+    auto settings = makeQSettings();
+    settings.setValue(numberOfLogsToKeepC(), number);
+}
+
+void ConfigFile::configureHttpLogging(std::optional<bool> enable)
+{
+    if (enable == std::nullopt) {
+        enable = logHttp();
     }
-}
 
-void ConfigFile::setLogHttp(bool b)
-{
-    QSettings settings(configFile(), QSettings::IniFormat);
-    settings.setValue(logHttpC(), b);
-    const QSet<QString> rule = { QStringLiteral("sync.httplogger=true") };
-    if (b) {
+    auto settings = makeQSettings();
+    settings.setValue(logHttpC(), enable.value());
+
+    static const QSet<QString> rule = { QStringLiteral("sync.httplogger=true") };
+
+    if (enable.value()) {
         Logger::instance()->addLogRule(rule);
     } else {
         Logger::instance()->removeLogRule(rule);
@@ -829,31 +814,32 @@ void ConfigFile::setLogHttp(bool b)
 
 bool ConfigFile::logHttp() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(logHttpC(), false).toBool();
 }
 
 bool ConfigFile::showExperimentalOptions() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(showExperimentalOptionsC(), false).toBool();
 }
 
-QString ConfigFile::clientVersionString() const
+QString ConfigFile::clientVersionWithBuildNumberString() const
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     return settings.value(clientVersionC(), QString()).toString();
 }
 
-void ConfigFile::setClientVersionString(const QString &version)
+void ConfigFile::setClientVersionWithBuildNumberString(const QString &version)
 {
-    QSettings settings(configFile(), QSettings::IniFormat);
+    auto settings = makeQSettings();
     settings.setValue(clientVersionC(), version);
 }
 
-std::unique_ptr<QSettings> ConfigFile::settingsWithGroup(const QString &group, QObject *parent)
+std::unique_ptr<QSettings> ConfigFile::settingsWithGroup(const QString &group)
 {
-    std::unique_ptr<QSettings> settings(new QSettings(ConfigFile::configFile(), QSettings::IniFormat, parent));
+    // this actually works by move magic
+    std::unique_ptr<QSettings> settings(new QSettings(makeQSettings()));
     settings->beginGroup(group);
     return settings;
 }
