@@ -63,18 +63,21 @@ class PropagatorJob : public QObject
     Q_OBJECT
 
 public:
-    explicit PropagatorJob(OwncloudPropagator *propagator);
+    explicit PropagatorJob(OwncloudPropagator *propagator, const QString &path);
 
     enum AbortType {
         Synchronous,
         Asynchronous
     };
+    Q_ENUM(AbortType)
 
     enum JobState {
         NotYetStarted,
         Running,
         Finished
     };
+    Q_ENUM(JobState)
+
     JobState _state;
 
     enum JobParallelism {
@@ -87,6 +90,7 @@ public:
             are executed. */
         WaitForFinished,
     };
+    Q_ENUM(JobParallelism)
 
     virtual JobParallelism parallelism() { return FullParallelism; }
 
@@ -110,6 +114,8 @@ public:
      */
     void setAssociatedComposite(PropagatorCompositeJob *job) { _associatedComposite = job; }
 
+    const QString path() { return _path; }
+
 public slots:
     /*
      * Asynchronous abort requires emit of abortFinished() signal,
@@ -146,6 +152,9 @@ protected:
      * becoming composite jobs themselves.
      */
     PropagatorCompositeJob *_associatedComposite = nullptr;
+
+private:
+    QString _path;
 };
 
 /*
@@ -162,12 +171,14 @@ protected:
 
 public:
     PropagateItemJob(OwncloudPropagator *propagator, const SyncFileItemPtr &item)
-        : PropagatorJob(propagator)
+        : PropagatorJob(propagator, item->destination())
         , _item(item)
     {
     }
     ~PropagateItemJob() override;
     bool scheduleSelfOrChild() override;
+
+    const SyncFileItem &item() const { return *_item.data(); }
 public slots:
     virtual void start() = 0;
 };
@@ -180,17 +191,7 @@ class PropagatorCompositeJob : public PropagatorJob
 {
     Q_OBJECT
 public:
-    QVector<PropagatorJob *> _jobsToDo;
-    SyncFileItemSet _tasksToDo;
-    QVector<PropagatorJob *> _runningJobs;
-    SyncFileItem::Status _hasError; // NoStatus,  or NormalError / SoftError if there was an error
-    quint64 _abortsCount;
-
-    explicit PropagatorCompositeJob(OwncloudPropagator *propagator)
-        : PropagatorJob(propagator)
-        , _hasError(SyncFileItem::NoStatus), _abortsCount(0)
-    {
-    }
+    explicit PropagatorCompositeJob(OwncloudPropagator *propagator, const QString &path);
 
     ~PropagatorCompositeJob() override
     {
@@ -231,6 +232,11 @@ public:
 
     qint64 committedDiskSpace() const override;
 
+    QMap<QString, SyncFileItem::Status> errorPaths() const { return _errorPaths; }
+
+    const QVector<PropagatorJob *> &jobsToDo() { return _jobsToDo; }
+    void setJobsToDo(const QVector<PropagatorJob *> &todo) { _jobsToDo = todo; }
+
 private slots:
     void slotSubJobAbortFinished();
     bool possiblyRunNextJob(PropagatorJob *next)
@@ -243,6 +249,13 @@ private slots:
 
     void slotSubJobFinished(SyncFileItem::Status status);
     void finalize();
+
+private:
+    QVector<PropagatorJob *> _jobsToDo;
+    SyncFileItemSet _tasksToDo;
+    QVector<PropagatorJob *> _runningJobs;
+    QMap<QString, SyncFileItem::Status> _errorPaths; // NoStatus,  or NormalError / SoftError if there was an error
+    quint64 _abortsCount = 0;
 };
 
 /**
@@ -254,7 +267,7 @@ class OWNCLOUDSYNC_EXPORT PropagateDirectory : public PropagateItemJob
     Q_OBJECT
 public:
     // e.g: create the directory
-    QScopedPointer<PropagateItemJob> _firstJob;
+    std::unique_ptr<PropagateItemJob> _firstJob;
 
     PropagatorCompositeJob _subJobs;
 
@@ -317,8 +330,6 @@ class OWNCLOUDSYNC_EXPORT PropagateRootDirectory : public PropagateDirectory
 {
     Q_OBJECT
 public:
-    PropagatorCompositeJob _dirDeletionJobs;
-
     explicit PropagateRootDirectory(OwncloudPropagator *propagator);
 
     bool scheduleSelfOrChild() override;
@@ -327,9 +338,15 @@ public:
 
     qint64 committedDiskSpace() const override;
 
+    void addDeleteJob(PropagatorJob *job);
+
 private slots:
     void slotSubJobsFinished(SyncFileItem::Status status) override;
     void slotDirDeletionJobsFinished(SyncFileItem::Status status);
+
+private:
+    PropagatorCompositeJob _dirDeletionJobs;
+    SyncFileItem::Status _status = SyncFileItem::NoStatus;
 };
 
 /**
@@ -348,11 +365,11 @@ public:
     {
         SyncFileItem::Status status = _item->_status;
         if (status == SyncFileItem::NoStatus) {
-            if (_item->_instruction == CSYNC_INSTRUCTION_ERROR) {
+            if (_item->instruction() == CSYNC_INSTRUCTION_ERROR) {
                 status = SyncFileItem::NormalError;
             } else {
                 status = SyncFileItem::FileIgnored;
-                OC_ASSERT(_item->_instruction == CSYNC_INSTRUCTION_IGNORE);
+                OC_ASSERT(_item->instruction() == CSYNC_INSTRUCTION_IGNORE);
             }
         }
         done(status, _item->_errorString);
@@ -367,16 +384,17 @@ public:
     bool _finishedEmited; // used to ensure that finished is only emitted once
 
 public:
-    OwncloudPropagator(AccountPtr account, const QString &localDir,
+    OwncloudPropagator(AccountPtr account, const SyncOptions &options, const QUrl &baseUrl, const QString &localDir,
         const QString &remoteFolder, SyncJournalDb *progressDb)
         : _journal(progressDb)
         , _finishedEmited(false)
-        , _bandwidthManager(this)
         , _anotherSyncNeeded(false)
-        , _chunkSize(10 * 1000 * 1000) // 10 MB, overridden in setSyncOptions
+        , _chunkSize(options._initialChunkSize)
         , _account(account)
+        , _syncOptions(options)
         , _localDir((localDir.endsWith(QLatin1Char('/'))) ? localDir : localDir + QLatin1Char('/'))
         , _remoteFolder((remoteFolder.endsWith(QLatin1Char('/'))) ? remoteFolder : remoteFolder + QLatin1Char('/'))
+        , _webDavUrl(baseUrl)
     {
         qRegisterMetaType<PropagatorJob::AbortType>("PropagatorJob::AbortType");
     }
@@ -386,11 +404,8 @@ public:
     void start(SyncFileItemSet &&_syncedItems);
 
     const SyncOptions &syncOptions() const;
-    void setSyncOptions(const SyncOptions &syncOptions);
 
-    int _downloadLimit = 0;
-    int _uploadLimit = 0;
-    BandwidthManager _bandwidthManager;
+    QPointer<BandwidthManager> _bandwidthManager;
 
     bool _abortRequested = false;
 
@@ -468,36 +483,21 @@ public:
     void reportProgress(const SyncFileItem &, qint64 bytes);
     void reportFileTotal(const SyncFileItem &item, qint64 newSize);
 
-    void abort()
-    {
-        if (_abortRequested)
-            return;
-        if (_rootJob) {
-            // Connect to abortFinished  which signals that abort has been asynchronously finished
-            connect(_rootJob.data(), &PropagateDirectory::abortFinished, this, &OwncloudPropagator::emitFinished);
-
-            // Use Queued Connection because we're possibly already in an item's finished stack
-            QMetaObject::invokeMethod(
-                _rootJob.data(), [this] {
-                    _rootJob->abort(PropagatorJob::AbortType::Asynchronous);
-                },
-                Qt::QueuedConnection);
-
-            // Give asynchronous abort 5000 msec to finish on its own
-            QTimer::singleShot(5000, this, &OwncloudPropagator::abortTimeout);
-        } else {
-            // No root job, call emitFinished
-            emitFinished(SyncFileItem::NormalError);
-        }
-    }
+    void abort();
 
     AccountPtr account() const;
 
+    QUrl webDavUrl() const
+    {
+        return _webDavUrl;
+    }
+
     enum DiskSpaceResult {
         DiskSpaceOk,
         DiskSpaceFailure,
         DiskSpaceCritical
     };
+    Q_ENUM(DiskSpaceResult)
 
     /** Checks whether there's enough disk space available to complete
      *  all jobs that are currently running.
@@ -517,7 +517,8 @@ public:
         PropagatorCompositeJob *composite, QString *error);
 
     // Map original path (as in the DB) to target final path
-    QMap<QString, QString> _renamedDirectories;
+    // TODO: no public members...
+    QHash<QString, QString> _renamedDirectories;
     QString adjustRenamedPath(const QString &original) const;
 
     /** Update the database for an item.
@@ -564,13 +565,6 @@ signals:
     /** Emitted when propagation has problems with a locked file. */
     void seenLockedFile(const QString &fileName, FileSystem::LockMode mode);
 
-    /** Emitted when propagation touches a file.
-     *
-     * Used to track our own file modifications such that notifications
-     * from the file watcher about these can be ignored.
-     */
-    void touchedFile(const QString &fileName);
-
     void insufficientLocalStorage();
     void insufficientRemoteStorage();
 
@@ -582,6 +576,7 @@ private:
 
     const QString _localDir; // absolute path to the local directory. ends with '/'
     const QString _remoteFolder; // remote folder, ends with '/'
+    const QUrl _webDavUrl; // full webdav url, might be the same as in the account
 };
 
 /**
