@@ -16,15 +16,11 @@
 
 #include <QString>
 #include <QVariant>
-#ifndef TOKEN_AUTH_ONLY
 #include <QPixmap>
 #include <QIcon>
-#endif
 #include <QCoreApplication>
 
-#include "config.h"
 #include "common/utility.h"
-#include "version.h"
 
 namespace OCC {
 
@@ -33,7 +29,6 @@ ownCloudTheme::ownCloudTheme()
 {
 }
 
-#ifndef TOKEN_AUTH_ONLY
 QColor ownCloudTheme::wizardHeaderBackgroundColor() const
 {
     return QColor(4, 30, 66);
@@ -44,11 +39,6 @@ QColor ownCloudTheme::wizardHeaderTitleColor() const
     return Qt::white;
 }
 
-QColor ownCloudTheme::wizardHeaderSubTitleColor() const
-{
-    return QColor(78, 133, 200);
-}
-
 QIcon ownCloudTheme::wizardHeaderLogo() const
 {
     return themeUniversalIcon(QStringLiteral("wizard_logo"));
@@ -58,6 +48,4 @@ QIcon ownCloudTheme::aboutIcon() const
 {
     return themeUniversalIcon(QStringLiteral("oc-image-about"));
 }
-
-#endif
 }
