@@ -12,28 +12,33 @@
  * for more details.
  */
 
-#include "config.h"
 #include "propagateupload.h"
-#include "owncloudpropagator_p.h"
-#include "networkjobs.h"
 #include "account.h"
+#include "filesystem.h"
+#include "networkjobs.h"
+#include "owncloudpropagator_p.h"
+#include "propagateremotedelete.h"
+#include "propagatorjobs.h"
+#include "syncengine.h"
+
+#include "common/asserts.h"
+#include "common/checksums.h"
 #include "common/syncjournaldb.h"
 #include "common/syncjournalfilerecord.h"
 #include "common/utility.h"
-#include "filesystem.h"
-#include "propagatorjobs.h"
-#include "common/checksums.h"
-#include "syncengine.h"
-#include "propagateremotedelete.h"
-#include "common/asserts.h"
 
-#include <QNetworkAccessManager>
-#include <QFileInfo>
+#include "libsync/theme.h"
+
 #include <QDir>
+#include <QFileInfo>
 #include <QJsonDocument>
 #include <QJsonObject>
+#include <QNetworkAccessManager>
+
+#include <chrono>
 #include <cmath>
-#include <cstring>
+
+using namespace std::chrono_literals;
 
 namespace OCC {
 
@@ -54,25 +59,17 @@ Q_LOGGING_CATEGORY(lcPropagateUploadNG, "sync.propagator.upload.ng", QtInfoMsg)
 static bool fileIsStillChanging(const SyncFileItem &item)
 {
     const QDateTime modtime = Utility::qDateTimeFromTime_t(item._modtime);
-    const qint64 msSinceMod = modtime.msecsTo(QDateTime::currentDateTimeUtc());
+    const auto secondsSinceMod = std::chrono::seconds(modtime.secsTo(QDateTime::currentDateTimeUtc()));
 
-    return std::chrono::milliseconds(msSinceMod) < SyncEngine::minimumFileAgeForUpload
+    return secondsSinceMod < SyncEngine::minimumFileAgeForUpload
         // if the mtime is too much in the future we *do* upload the file
-        && msSinceMod > -10000;
+        && secondsSinceMod > -1s;
 }
 
-PUTFileJob::PUTFileJob(AccountPtr account, const QString &path, std::unique_ptr<QIODevice> device, const QMap<QByteArray, QByteArray> &headers, int chunk, QObject *parent)
-    : PUTFileJob(account, Utility::concatUrlPath(account->davUrl(), path), std::move(device), headers, chunk, parent)
-{
-    // Long uploads must not block non-propagation jobs.
-    setPriority(QNetworkRequest::LowPriority);
-}
-
-PUTFileJob::PUTFileJob(AccountPtr account, const QUrl &url, std::unique_ptr<QIODevice> device, const QMap<QByteArray, QByteArray> &headers, int chunk, QObject *parent)
-    : AbstractNetworkJob(account, QString(), parent)
+PUTFileJob::PUTFileJob(AccountPtr account, const QUrl &url, const QString &path, std::unique_ptr<QIODevice> &&device, const QMap<QByteArray, QByteArray> &headers, int chunk, QObject *parent)
+    : AbstractNetworkJob(account, url, path, parent)
     , _device(device.release())
     , _headers(headers)
-    , _url(url)
     , _chunk(chunk)
 {
     _device->setParent(this);
@@ -82,8 +79,6 @@ PUTFileJob::PUTFileJob(AccountPtr account, const QUrl &url, std::unique_ptr<QIOD
 
 PUTFileJob::~PUTFileJob()
 {
-    // Make sure that we destroy the QNetworkReply before our _device of which it keeps an internal pointer.
-    setReply(nullptr);
 }
 
 void PUTFileJob::start()
@@ -92,24 +87,18 @@ void PUTFileJob::start()
     for (auto it = _headers.cbegin(); it != _headers.cend(); ++it) {
         req.setRawHeader(it.key(), it.value());
     }
-    sendRequest("PUT", _url, req, _device);
-
-    connect(this, &AbstractNetworkJob::networkActivity, account().data(), &Account::propagatorNetworkActivity);
+    sendRequest("PUT", req, _device);
     _requestTimer.start();
     AbstractNetworkJob::start();
 }
 
-bool PUTFileJob::finished()
+void PUTFileJob::finished()
 {
     _device->close();
 
-    qCInfo(lcPutJob) << "PUT of" << reply()->request().url().toString() << "FINISHED WITH STATUS"
-                     << replyStatusString()
+    qCInfo(lcPutJob) << "PUT of" << reply()->request().url().toString() << "FINISHED WITH STATUS" << replyStatusString()
                      << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt()
-                     << reply()->attribute(QNetworkRequest::HttpReasonPhraseAttribute);
-
-    emit finishedSignal();
-    return true;
+                     << reply()->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString();
 }
 
 void PUTFileJob::newReplyHook(QNetworkReply *reply)
@@ -117,6 +106,11 @@ void PUTFileJob::newReplyHook(QNetworkReply *reply)
     connect(reply, &QNetworkReply::uploadProgress, this, &PUTFileJob::uploadProgress);
 }
 
+const QString PropagateUploadFileCommon::fileChangedMessage()
+{
+    return tr("Local file changed during sync. It will be resumed.");
+}
+
 void PropagateUploadFileCommon::setDeleteExisting(bool enabled)
 {
     _deleteExisting = enabled;
@@ -152,12 +146,11 @@ void PropagateUploadFileCommon::start()
         return slotComputeContentChecksum();
     }
 
-    auto job = new DeleteJob(propagator()->account(),
+    auto job = new DeleteJob(propagator()->account(), propagator()->webDavUrl(),
         propagator()->fullRemotePath(_item->_file),
         this);
-    _jobs.append(job);
+    addChildJob(job);
     connect(job, &DeleteJob::finishedSignal, this, &PropagateUploadFileCommon::slotComputeContentChecksum);
-    connect(job, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
     job->start();
 }
 
@@ -173,13 +166,19 @@ void PropagateUploadFileCommon::slotComputeContentChecksum()
     // change during the checksum calculation
     _item->_modtime = FileSystem::getModTime(filePath);
 
-    const QByteArray checksumType = propagator()->account()->capabilities().preferredUploadChecksumType();
+    const auto checksumType = propagator()->account()->capabilities().preferredUploadChecksumType();
 
     // Maybe the discovery already computed the checksum?
-    QByteArray existingChecksumType, existingChecksum;
-    parseChecksumHeader(_item->_checksumHeader, &existingChecksumType, &existingChecksum);
-    if (existingChecksumType == checksumType) {
-        slotComputeTransmissionChecksum(checksumType, existingChecksum);
+    const auto checksumHeader = ChecksumHeader::parseChecksumHeader(_item->_checksumHeader);
+    if (checksumHeader.type() == checksumType) {
+        slotComputeTransmissionChecksum(checksumType, checksumHeader.checksum());
+        return;
+    }
+
+    // we must be able to read the file
+    if (FileSystem::isFileLocked(filePath, FileSystem::LockMode::SharedRead)) {
+        emit propagator()->seenLockedFile(filePath, FileSystem::LockMode::SharedRead);
+        abortWithError(SyncFileItem::SoftError, tr("%1 the file is currently in use").arg(filePath));
         return;
     }
 
@@ -194,9 +193,9 @@ void PropagateUploadFileCommon::slotComputeContentChecksum()
     computeChecksum->start(filePath);
 }
 
-void PropagateUploadFileCommon::slotComputeTransmissionChecksum(const QByteArray &contentChecksumType, const QByteArray &contentChecksum)
+void PropagateUploadFileCommon::slotComputeTransmissionChecksum(CheckSums::Algorithm contentChecksumType, const QByteArray &contentChecksum)
 {
-    _item->_checksumHeader = makeChecksumHeader(contentChecksumType, contentChecksum);
+    _item->_checksumHeader = ChecksumHeader(contentChecksumType, contentChecksum).makeChecksumHeader();
 
     // Reuse the content checksum as the transmission checksum if possible
     const auto supportedTransmissionChecksums =
@@ -206,29 +205,36 @@ void PropagateUploadFileCommon::slotComputeTransmissionChecksum(const QByteArray
         return;
     }
 
+    const QString filePath = propagator()->fullLocalPath(_item->_file);
+    // we must be able to read the file
+    if (FileSystem::isFileLocked(filePath, FileSystem::LockMode::SharedRead)) {
+        emit propagator()->seenLockedFile(filePath, FileSystem::LockMode::SharedRead);
+        abortWithError(SyncFileItem::SoftError, tr("%1 the file is currently in use").arg(filePath));
+        return;
+    }
+
     // Compute the transmission checksum.
     auto computeChecksum = new ComputeChecksum(this);
     if (uploadChecksumEnabled()) {
         computeChecksum->setChecksumType(propagator()->account()->capabilities().uploadChecksumType());
     } else {
-        computeChecksum->setChecksumType(QByteArray());
+        computeChecksum->setChecksumType(CheckSums::Algorithm::PARSE_ERROR);
     }
 
     connect(computeChecksum, &ComputeChecksum::done,
         this, &PropagateUploadFileCommon::slotStartUpload);
     connect(computeChecksum, &ComputeChecksum::done,
         computeChecksum, &QObject::deleteLater);
-    const QString filePath = propagator()->fullLocalPath(_item->_file);
     computeChecksum->start(filePath);
 }
 
-void PropagateUploadFileCommon::slotStartUpload(const QByteArray &transmissionChecksumType, const QByteArray &transmissionChecksum)
+void PropagateUploadFileCommon::slotStartUpload(CheckSums::Algorithm transmissionChecksumType, const QByteArray &transmissionChecksum)
 {
     // Remove ourselfs from the list of active job, before any posible call to done()
     // When we start chunks, we will add it again, once for every chunks.
     propagator()->_activeJobList.removeOne(this);
 
-    _transmissionChecksumHeader = makeChecksumHeader(transmissionChecksumType, transmissionChecksum);
+    _transmissionChecksumHeader = ChecksumHeader(transmissionChecksumType, transmissionChecksum).makeChecksumHeader();
 
     // If no checksum header was not set, reuse the transmission checksum as the content checksum.
     if (_item->_checksumHeader.isEmpty()) {
@@ -241,7 +247,7 @@ void PropagateUploadFileCommon::slotStartUpload(const QByteArray &transmissionCh
         done(SyncFileItem::SoftError, tr("File Removed"));
         return;
     }
-    _item->_size = FileSystem::getSize(fullFilePath);
+    _item->_size = FileSystem::getSize(QFileInfo{fullFilePath});
 
     const time_t prevModtime = _item->_modtime; // the _item value was set in PropagateUploadFile::start()
     // but a potential checksum calculation could have taken some time during which the file could
@@ -253,7 +259,7 @@ void PropagateUploadFileCommon::slotStartUpload(const QByteArray &transmissionCh
     _item->_modtime = FileSystem::getModTime(fullFilePath);
     if (prevModtime != _item->_modtime || fileIsStillChanging(*_item)) {
         propagator()->_anotherSyncNeeded = true;
-        done(SyncFileItem::Message, tr("Local file changed during sync. It will be resumed."));
+        done(SyncFileItem::Message, fileChangedMessage());
         return;
     }
 
@@ -271,7 +277,9 @@ UploadDevice::UploadDevice(const QString &fileName, qint64 start, qint64 size, B
     , _bandwidthLimited(false)
     , _choked(false)
 {
-    _bandwidthManager->registerUploadDevice(this);
+    if (_bandwidthManager) {
+        _bandwidthManager->registerUploadDevice(this);
+    }
 }
 
 
@@ -289,7 +297,7 @@ bool UploadDevice::open(QIODevice::OpenMode mode)
 
     // Get the file size now: _file.fileName() is no longer reliable
     // on all platforms after openAndSeekFileSharedRead().
-    auto fileDiskSize = FileSystem::getSize(_file.fileName());
+    auto fileDiskSize = FileSystem::getSize(QFileInfo{_file.fileName()});
 
     QString openError;
     if (!FileSystem::openAndSeekFileSharedRead(&_file, &openError, _start)) {
@@ -395,21 +403,23 @@ void UploadDevice::giveBandwidthQuota(qint64 bwq)
 {
     if (!atEnd()) {
         _bandwidthQuota = bwq;
-        QMetaObject::invokeMethod(this, "readyRead", Qt::QueuedConnection); // tell QNAM that we have quota
+        QMetaObject::invokeMethod(this, &UploadDevice::readyRead, Qt::QueuedConnection); // tell QNAM that we have quota
     }
 }
 
 void UploadDevice::setBandwidthLimited(bool b)
 {
-    _bandwidthLimited = b;
-    QMetaObject::invokeMethod(this, "readyRead", Qt::QueuedConnection);
+    if (_bandwidthLimited != b) {
+        _bandwidthLimited = b;
+        QMetaObject::invokeMethod(this, &UploadDevice::readyRead, Qt::QueuedConnection);
+    }
 }
 
 void UploadDevice::setChoked(bool b)
 {
     _choked = b;
     if (!_choked) {
-        QMetaObject::invokeMethod(this, "readyRead", Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, &UploadDevice::readyRead, Qt::QueuedConnection);
     }
 }
 
@@ -482,17 +492,13 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
 
 void PropagateUploadFileCommon::adjustLastJobTimeout(AbstractNetworkJob *job, qint64 fileSize)
 {
-    job->setTimeout(qBound(
-        job->timeoutMsec(),
-        // Calculate 3 minutes for each gigabyte of data
-        qint64((3 * 60 * 1000) * fileSize / 1e9),
-        // Maximum of 30 minutes
-        30 * 60 * 1000LL));
-}
-
-void PropagateUploadFileCommon::slotJobDestroyed(QObject *job)
-{
-    _jobs.erase(std::remove(_jobs.begin(), _jobs.end(), job), _jobs.end());
+    // Calculate 3 minutes for each gigabyte of data
+    const auto timeout = std::chrono::minutes(static_cast<quint64>((3min).count() * fileSize / 1e9));
+    // Maximum of 30 minutes
+    // not less than the default/current val
+    if (timeout > job->timeoutSec() && timeout < 30min) {
+        job->setTimeout(timeout);
+    }
 }
 
 // This function is used whenever there is an error occuring and jobs might be in progress
@@ -505,15 +511,23 @@ void PropagateUploadFileCommon::abortWithError(SyncFileItem::Status status, cons
     }
 }
 
+void PropagateUploadFileCommon::addChildJob(AbstractNetworkJob *job)
+{
+    _childJobs.insert(job);
+    connect(
+        job, &AbstractNetworkJob::aboutToFinishSignal, this, [job, this] {
+            _childJobs.erase(job);
+        },
+        Qt::DirectConnection);
+}
+
 QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
 {
     QMap<QByteArray, QByteArray> headers;
     headers[QByteArrayLiteral("Content-Type")] = QByteArrayLiteral("application/octet-stream");
     headers[QByteArrayLiteral("X-OC-Mtime")] = QByteArray::number(qint64(_item->_modtime));
-    if (qEnvironmentVariableIntValue("OWNCLOUD_LAZYOPS"))
-        headers[QByteArrayLiteral("OC-LazyOps")] = QByteArrayLiteral("true");
 
-    if (_item->_file.contains(QLatin1String(".sys.admin#recall#"))) {
+    if (Q_UNLIKELY(Theme::instance()->enableCernBranding() && _item->_file.contains(QLatin1String(".sys.admin#recall#")))) {
         // This is a file recall triggered by the admin.  Note: the
         // recall list file created by the admin and downloaded by the
         // client (.sys.admin#recall#) also falls into this category
@@ -524,13 +538,12 @@ QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
         headers["OC-Tag"] = ".sys.admin#recall#";
     }
 
-    if (!_item->_etag.isEmpty() && _item->_etag != "empty_etag"
-        && _item->_instruction != CSYNC_INSTRUCTION_NEW // On new files never send a If-Match
-        && _item->_instruction != CSYNC_INSTRUCTION_TYPE_CHANGE
+    if (!_item->_etag.isEmpty() && _item->_etag != QLatin1String("empty_etag")
+        && (_item->instruction() & ~(CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_TYPE_CHANGE)) // On new files never send a If-Match
         && !_deleteExisting) {
         // We add quotes because the owncloud server always adds quotes around the etag, and
         //  csync_owncloud.c's owncloud_file_id always strips the quotes.
-        headers[QByteArrayLiteral("If-Match")] = '"' + _item->_etag + '"';
+        headers[QByteArrayLiteral("If-Match")] = QStringLiteral("\"%1\"").arg(_item->_etag).toUtf8();
     }
 
     // Set up a conflict file header pointing to the original file
@@ -552,47 +565,42 @@ QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
 
 void PropagateUploadFileCommon::finalize()
 {
+    // Update the quota, if known
+    if (!_quotaUpdated) {
+        auto quotaIt = propagator()->_folderQuota.find(QFileInfo(_item->_file).path());
+        if (quotaIt != propagator()->_folderQuota.end()) {
+            quotaIt.value() -= _item->_size;
+        }
+        _quotaUpdated = true;
+    }
+
     if (_item->_remotePerm.isNull()) {
         qCWarning(lcPropagateUpload) << "PropagateUploadFileCommon::finalize: Missing permissions for" << propagator()->fullRemotePath(_item->_file);
-        auto permCheck = new PropfindJob(propagator()->account(), propagator()->fullRemotePath(_item->_file));
-        _jobs.append(permCheck);
+        auto permCheck = new PropfindJob(propagator()->account(), propagator()->webDavUrl(), propagator()->fullRemotePath(_item->_file), PropfindJob::Depth::Zero, this);
+        addChildJob(permCheck);
         permCheck->setProperties({ "http://owncloud.org/ns:permissions" });
-        connect(permCheck, &PropfindJob::result, this, [this, permCheck](const QMap<QString, QString> &map) {
+        connect(permCheck, &PropfindJob::directoryListingIterated, this, [this](const QString &, const QMap<QString, QString> &map) {
             _item->_remotePerm = RemotePermissions::fromServerString(map.value(QStringLiteral("permissions")));
             finalize();
-            slotJobDestroyed(permCheck);
         });
-        connect(permCheck, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
         permCheck->start();
         return;
     }
-    // Update the quota, if known
-    auto quotaIt = propagator()->_folderQuota.find(QFileInfo(_item->_file).path());
-    if (quotaIt != propagator()->_folderQuota.end())
-        quotaIt.value() -= _item->_size;
 
-
-#ifdef Q_OS_WIN
-    m_fileLock.close();
-#endif
     // Update the database entry
     const auto result = propagator()->updateMetadata(*_item);
     if (!result) {
         done(SyncFileItem::FatalError, tr("Error updating metadata: %1").arg(result.error()));
         return;
-    } else if (result.get() == Vfs::ConvertToPlaceholderResult::Locked) {
-        done(SyncFileItem::SoftError, tr("The file %1 is currently in use").arg(_item->_file));
-        return;
     }
 
     // Files that were new on the remote shouldn't have online-only pin state
     // even if their parent folder is online-only.
-    if (_item->_instruction == CSYNC_INSTRUCTION_NEW
-        || _item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE) {
+    if (_item->instruction() & (CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_TYPE_CHANGE)) {
         auto &vfs = propagator()->syncOptions()._vfs;
         const auto pin = vfs->pinState(_item->_file);
         if (pin && *pin == PinState::OnlineOnly) {
-            vfs->setPinState(_item->_file, PinState::Unspecified);
+            std::ignore = vfs->setPinState(_item->_file, PinState::Unspecified);
         }
     }
 
@@ -622,7 +630,9 @@ void PropagateUploadFileCommon::abortNetworkJobs(
     };
 
     // Abort all running jobs, except for explicitly excluded ones
-    for (auto *job : qAsConst(_jobs)) {
+    // perform actions on a copy as aborted jobs will be removed from _childJobs
+    const auto children = _childJobs;
+    for (auto *job : children) {
         auto reply = job->reply();
         if (!reply || !reply->isRunning())
             continue;
@@ -642,7 +652,7 @@ void PropagateUploadFileCommon::abortNetworkJobs(
             // Connect to finished signal of job reply to asynchonously finish the abort
             connect(reply, &QNetworkReply::finished, this, oneAbortFinished);
         }
-        reply->abort();
+        job->abort();
     }
 
     if (*runningCount == 0 && abortType == AbortType::Asynchronous)
