@@ -13,19 +13,19 @@
  */
 
 #include "theme.h"
-#include "config.h"
 #include "common/depreaction.h"
 #include "common/utility.h"
-#include "version.h"
-#include "configfile.h"
+#include "common/version.h"
 #include "common/vfs.h"
+#include "config.h"
+#include "configfile.h"
+
+#include "resources/resources.h"
 
 #include <QtCore>
-#ifndef TOKEN_AUTH_ONLY
 #include <QtGui>
 #include <QStyle>
 #include <QApplication>
-#endif
 #include <QSslSocket>
 
 #include "owncloudtheme.h"
@@ -65,15 +65,9 @@ QString blackTheme()
     return QStringLiteral("black");
 }
 
-constexpr bool strings_equal(char const *a, char const *b)
-{
-    return *a == *b && (*a == '\0' || strings_equal(a + 1, b + 1));
-}
-
 constexpr bool isVanilla()
 {
-    // TODO: c++17 stringview
-    return strings_equal(APPLICATION_SHORTNAME, "ownCloud");
+    return std::string_view(APPLICATION_SHORTNAME) == "ownCloud";
 }
 }
 namespace OCC {
@@ -92,45 +86,6 @@ Theme::~Theme()
 {
 }
 
-QString Theme::statusHeaderText(SyncResult::Status status) const
-{
-    QString resultStr;
-
-    switch (status) {
-    case SyncResult::Undefined:
-        resultStr = QCoreApplication::translate("theme", "Status undefined");
-        break;
-    case SyncResult::NotYetStarted:
-        resultStr = QCoreApplication::translate("theme", "Waiting to start sync");
-        break;
-    case SyncResult::SyncRunning:
-        resultStr = QCoreApplication::translate("theme", "Sync is running");
-        break;
-    case SyncResult::Success:
-        resultStr = QCoreApplication::translate("theme", "Sync Success");
-        break;
-    case SyncResult::Problem:
-        resultStr = QCoreApplication::translate("theme", "Sync Success, some files were ignored.");
-        break;
-    case SyncResult::Error:
-        resultStr = QCoreApplication::translate("theme", "Sync Error");
-        break;
-    case SyncResult::SetupError:
-        resultStr = QCoreApplication::translate("theme", "Setup Error");
-        break;
-    case SyncResult::SyncPrepare:
-        resultStr = QCoreApplication::translate("theme", "Preparing to sync");
-        break;
-    case SyncResult::SyncAbortRequested:
-        resultStr = QCoreApplication::translate("theme", "Aborting...");
-        break;
-    case SyncResult::Paused:
-        resultStr = QCoreApplication::translate("theme", "Sync is paused");
-        break;
-    }
-    return resultStr;
-}
-
 QString Theme::appNameGUI() const
 {
     return QStringLiteral(APPLICATION_NAME);
@@ -141,9 +96,19 @@ QString Theme::appName() const
     return QStringLiteral(APPLICATION_SHORTNAME);
 }
 
-QString Theme::version() const
+QString Theme::appDotVirtualFileSuffix() const
 {
-    return QStringLiteral(MIRALL_VERSION_STRING);
+    return QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
+}
+
+QString Theme::orgDomainName() const
+{
+    return QStringLiteral(APPLICATION_REV_DOMAIN);
+}
+
+QString Theme::vendor() const
+{
+    return QStringLiteral(APPLICATION_VENDOR);
 }
 
 QString Theme::configFileName() const
@@ -151,11 +116,14 @@ QString Theme::configFileName() const
     return QStringLiteral(APPLICATION_EXECUTABLE ".cfg");
 }
 
-#ifndef TOKEN_AUTH_ONLY
-
 QIcon Theme::applicationIcon() const
 {
-    return themeUniversalIcon(QStringLiteral(APPLICATION_ICON_NAME "-icon"));
+    return themeUniversalIcon(applicationIconName() + QStringLiteral("-icon"));
+}
+
+QString Theme::applicationIconName() const
+{
+    return QStringLiteral(APPLICATION_ICON_NAME);
 }
 
 QIcon Theme::aboutIcon() const
@@ -163,13 +131,6 @@ QIcon Theme::aboutIcon() const
     return applicationIcon();
 }
 
-bool Theme::isUsingDarkTheme() const
-{
-    //TODO: replace by a command line switch
-    static bool forceDark = qEnvironmentVariableIntValue("OWNCLOUD_FORCE_DARK_MODE") != 0;
-    return forceDark || QPalette().base().color().lightnessF() <= 0.5;
-}
-
 bool Theme::allowDarkTheme() const
 {
     return _hasBrandedColored == _hasBrandedDark;
@@ -194,7 +155,7 @@ QIcon Theme::themeTrayIcon(const QString &name, bool sysTrayMenuVisible, IconTyp
 
 QIcon Theme::themeIcon(const QString &name, Theme::IconType iconType) const
 {
-    return loadIcon((isUsingDarkTheme() && allowDarkTheme()) ? darkTheme() : coloredTheme(), name, iconType);
+    return loadIcon((Resources::isUsingDarkTheme() && allowDarkTheme()) ? darkTheme() : coloredTheme(), name, iconType);
 }
 /*
  * helper to load a icon from either the icon theme the desktop provides or from
@@ -231,7 +192,7 @@ QIcon Theme::loadIcon(const QString &flavor, const QString &name, IconType iconT
                 cached.addFile(pixmapName, { size, size });
             } else if (size >= 128) {
                 if (!previousIcon.isEmpty()) {
-                    qWarning() << "Upsacling:" << previousIcon << "to" << size;
+                    qWarning() << "Upscaling:" << previousIcon << "to" << size;
                     cached.addPixmap(QPixmap(previousIcon).scaled({ size, size }, Qt::KeepAspectRatio, Qt::SmoothTransformation));
                 }
             }
@@ -275,8 +236,6 @@ QString Theme::systrayIconFlavor(bool mono, bool sysTrayMenuVisible) const
     return flavor;
 }
 
-#endif
-
 Theme::Theme()
     : QObject(nullptr)
 {
@@ -301,7 +260,7 @@ QString Theme::defaultServerFolder() const
 
 QString Theme::helpUrl() const
 {
-    return QStringLiteral("https://doc.owncloud.org/desktop/%1.%2/").arg(MIRALL_VERSION_MAJOR).arg(MIRALL_VERSION_MINOR);
+    return QStringLiteral("https://doc.owncloud.com/desktop/latest/");
 }
 
 QString Theme::conflictHelpUrl() const
@@ -335,12 +294,6 @@ QString Theme::forceConfigAuthType() const
     return QString();
 }
 
-
-QString Theme::defaultClientFolder() const
-{
-    return appName();
-}
-
 void Theme::setSystrayUseMonoIcons(bool mono)
 {
     _mono = mono;
@@ -359,9 +312,9 @@ bool Theme::monoIconsAvailable() const
     return hasTheme(IconType::BrandedIcon, systrayIconFlavor(true));
 }
 
-QString Theme::updateCheckUrl() const
+QUrl Theme::updateCheckUrl() const
 {
-    return QStringLiteral("https://updates.owncloud.com/client/");
+    return QUrl(QStringLiteral(APPLICATION_UPDATE_URL));
 }
 
 qint64 Theme::newBigFolderSizeLimit() const
@@ -370,30 +323,18 @@ qint64 Theme::newBigFolderSizeLimit() const
     return 500;
 }
 
-bool Theme::wizardHideExternalStorageConfirmationCheckbox() const
-{
-    return false;
-}
-
 bool Theme::wizardSkipAdvancedPage() const
 {
     return false;
 }
 
-bool Theme::wizardHideFolderSizeLimitCheckbox() const
-{
-    return false;
-}
-
 QString Theme::gitSHA1(VersionFormat format) const
 {
-#ifdef GIT_SHA1
-    const auto gitSha = QStringLiteral(GIT_SHA1);
-    const auto gitShahSort = gitSha.left(6);
+    const QString gitShahSort = Version::gitSha().left(6);
     if (!aboutShowCopyright()) {
         return gitShahSort;
     }
-    const auto gitUrl = QStringLiteral("https://github.com/owncloud/client/commit/%1").arg(gitSha);
+    const auto gitUrl = QStringLiteral("https://github.com/owncloud/client/commit/%1").arg(Version::gitSha());
     switch (format) {
     case Theme::VersionFormat::OneLiner:
         Q_FALLTHROUGH();
@@ -404,7 +345,6 @@ QString Theme::gitSHA1(VersionFormat format) const
     case Theme::VersionFormat::RichText:
         return QStringLiteral("<a href=\"%1\">%3</a>").arg(gitUrl, gitShahSort);
     }
-#endif
     return QString();
 }
 
@@ -424,31 +364,30 @@ QString Theme::aboutVersions(Theme::VersionFormat format) const
         Q_UNREACHABLE();
     }();
     const QString qtVersion = QString::fromUtf8(qVersion());
-    const QString qtVersionString = (QLatin1String(QT_VERSION_STR) == qtVersion ? qtVersion : QCoreApplication::translate("ownCloudTheme::qtVer", "%1 (Built against Qt %1)").arg(qtVersion, QStringLiteral(QT_VERSION_STR)));
-    QString _version = version();
+    const QString qtVersionString = (QLatin1String(QT_VERSION_STR) == qtVersion ? qtVersion : QCoreApplication::translate("ownCloudTheme::qtVer", "%1 (Built against Qt %2)").arg(qtVersion, QStringLiteral(QT_VERSION_STR)));
+    QString _version = Version::displayString();
     QString gitUrl;
-#ifdef GIT_SHA1
-    if (format != Theme::VersionFormat::Url) {
-        _version = QCoreApplication::translate("ownCloudTheme::versionWithSha", "%1 %2").arg(version(), gitSHA1(format));
-    } else {
-        gitUrl = gitSHA1(format) + br;
+    if (!Version::gitSha().isEmpty()) {
+        if (format != Theme::VersionFormat::Url) {
+            _version = QCoreApplication::translate("ownCloudTheme::versionWithSha", "%1 %2").arg(_version, gitSHA1(format));
+        } else {
+            gitUrl = gitSHA1(format) + br;
+        }
     }
-#endif
+    QStringList sysInfo = {QStringLiteral("OS: %1-%2").arg(QSysInfo::productType(), QSysInfo::kernelVersion())};
+    // may be called by both GUI and CLI, but we can display QPA only for the former
+    if (auto guiApp = qobject_cast<QGuiApplication *>(qApp)) {
+        sysInfo << QStringLiteral("QPA: %1").arg(guiApp->platformName());
+    }
+
     return QCoreApplication::translate("ownCloudTheme::aboutVersions()",
-        "%1 %2 %3%8"
-        "%9"
-        "Libraries Qt %4, %5%8"
-        "Using virtual files plugin: %6%8"
-        "%7")
-        .arg(appName(),
-            _version,
-            QStringLiteral(__DATE__ " " __TIME__),
-            qtVersionString,
-            QSslSocket::sslLibraryVersionString(),
-            Vfs::modeToString(bestAvailableVfsMode()),
-            QSysInfo::productType() % QLatin1Char('-') % QSysInfo::kernelVersion(),
-            br,
-            gitUrl);
+        "%1 %2%7"
+        "%8"
+        "Libraries Qt %3, %4%7"
+        "Using virtual files plugin: %5%7"
+        "%6")
+        .arg(appName(), _version, qtVersionString, QSslSocket::sslLibraryVersionString(),
+            Utility::enumToString(VfsPluginManager::instance().bestAvailableVfsMode()), sysInfo.join(br), br, gitUrl);
 }
 
 
@@ -458,19 +397,17 @@ QString Theme::about() const
     // changing the location of the settings and other registery keys.
     const QString vendor = isVanilla() ? QStringLiteral("ownCloud GmbH") : QStringLiteral(APPLICATION_VENDOR);
     return tr("<p>Version %1. For more information visit <a href=\"%2\">https://%3</a></p>"
-              "<p>For known issues and help, please visit: <a href=\"https://central.owncloud.org/c/desktop-client\">https://central.owncloud.org</a></p>"
+              "<p>For known issues and help, please visit: <a href=\"https://central.owncloud.com/c/desktop-client\">https://central.owncloud.com</a></p>"
               "<p><small>By Klaas Freitag, Daniel Molkentin, Olivier Goffart, Markus Götz, "
-              " Jan-Christoph Borchardt, Thomas Müller, Dominik Schmidt, Michael Stingl, Hannah von Reth, and others.</small></p>"
+              " Jan-Christoph Borchardt, Thomas Müller,<br>"
+              "Dominik Schmidt, Michael Stingl, Hannah von Reth, Fabian Müller and others.</small></p>"
               "<p>Copyright ownCloud GmbH</p>"
               "<p>Distributed by %4 and licensed under the GNU General Public License (GPL) Version 2.0.<br/>"
               "%5 and the %5 logo are registered trademarks of %4 in the "
               "United States, other countries, or both.</p>"
               "<p><small>%6</small></p>")
-        .arg(Utility::escape(version()),
-            Utility::escape(QStringLiteral("https://" APPLICATION_DOMAIN)),
-            Utility::escape(QStringLiteral(APPLICATION_DOMAIN)),
-            Utility::escape(vendor),
-            Utility::escape(appNameGUI()),
+        .arg(Utility::escape(Version::displayString()), Utility::escape(QStringLiteral("https://" APPLICATION_DOMAIN)),
+            Utility::escape(QStringLiteral(APPLICATION_DOMAIN)), Utility::escape(vendor), Utility::escape(appNameGUI()),
             aboutVersions(Theme::VersionFormat::RichText));
 }
 
@@ -479,65 +416,56 @@ bool Theme::aboutShowCopyright() const
     return true;
 }
 
-#ifndef TOKEN_AUTH_ONLY
-QIcon Theme::syncStateIcon(SyncResult::Status status, bool sysTray, bool sysTrayMenuVisible) const
+QString Theme::syncStateIconName(const SyncResult &result) const
 {
-    return syncStateIcon(SyncResult { status }, sysTray, sysTrayMenuVisible);
-}
-QIcon Theme::syncStateIcon(const SyncResult &result, bool sysTray, bool sysTrayMenuVisible) const
-{
-    QString statusIcon;
-
     switch (result.status()) {
     case SyncResult::NotYetStarted:
-        Q_FALLTHROUGH();
+        [[fallthrough]];
     case SyncResult::SyncRunning:
-        statusIcon = QStringLiteral("state-sync");
-        break;
+        return QStringLiteral("sync");
     case SyncResult::SyncAbortRequested:
-        Q_FALLTHROUGH();
+        [[fallthrough]];
     case SyncResult::Paused:
-        statusIcon = QStringLiteral("state-pause");
-        break;
+        return QStringLiteral("pause");
     case SyncResult::SyncPrepare:
-        Q_FALLTHROUGH();
+        [[fallthrough]];
     case SyncResult::Success:
-        if (result.hasUnresolvedConflicts()) {
-            return syncStateIcon(SyncResult { SyncResult::Problem }, sysTray, sysTrayMenuVisible);
+        if (!result.hasUnresolvedConflicts()) {
+            return QStringLiteral("ok");
         }
-        statusIcon = QStringLiteral("state-ok");
-        break;
+        [[fallthrough]];
     case SyncResult::Problem:
-        Q_FALLTHROUGH();
+        [[fallthrough]];
     case SyncResult::Undefined:
         // this can happen if no sync connections are configured.
-        statusIcon = QStringLiteral("state-information");
-        break;
+        return QStringLiteral("information");
+    case SyncResult::Offline:
+        return QStringLiteral("offline");
     case SyncResult::Error:
-        Q_FALLTHROUGH();
+        [[fallthrough]];
     case SyncResult::SetupError:
-    // FIXME: Use state-problem once we have an icon.
-        statusIcon = QStringLiteral("state-error");
-    }
-    if (sysTray) {
-        return themeTrayIcon(statusIcon, sysTrayMenuVisible);
-    } else {
-        return themeIcon(statusIcon);
+        // FIXME: Use problem once we have an icon.
+        return QStringLiteral("error");
     }
+    Q_UNREACHABLE();
 }
 
-QIcon Theme::folderDisabledIcon() const
+QIcon Theme::syncStateIcon(SyncResult::Status status, bool sysTray, bool sysTrayMenuVisible) const
 {
-    return themeIcon(QLatin1String("state-pause"));
+    return syncStateIcon(SyncResult{status}, sysTray, sysTrayMenuVisible);
 }
 
-QIcon Theme::folderOfflineIcon(bool sysTray, bool sysTrayMenuVisible) const
+QIcon Theme::syncStateIcon(const SyncResult &result, bool sysTray, bool sysTrayMenuVisible) const
+{
+    return syncStateIcon(QStringLiteral("state-%1").arg(syncStateIconName(result)), sysTray, sysTrayMenuVisible);
+}
+
+QIcon Theme::syncStateIcon(const QString &iconName, bool sysTray, bool sysTrayMenuVisible) const
 {
-    const auto statusIcon = QLatin1String("state-offline");
     if (sysTray) {
-        return themeTrayIcon(statusIcon, sysTrayMenuVisible);
+        return themeTrayIcon(iconName, sysTrayMenuVisible);
     } else {
-        return themeIcon(statusIcon);
+        return themeIcon(iconName);
     }
 }
 
@@ -561,17 +489,6 @@ QIcon Theme::wizardHeaderLogo() const
     return applicationIcon();
 }
 
-QPixmap Theme::wizardHeaderBanner(const QSize &size) const
-{
-    const QColor c = wizardHeaderBackgroundColor();
-    if (!c.isValid())
-        return QPixmap();
-    QPixmap pix(size);
-    pix.fill(c);
-    return pix;
-}
-#endif
-
 QString Theme::webDavPath() const
 {
     return QStringLiteral("remote.php/webdav/");
@@ -613,11 +530,6 @@ QString Theme::wizardUrlPostfix() const
     return QString();
 }
 
-QString Theme::wizardUrlHint() const
-{
-    return QString();
-}
-
 QString Theme::quotaBaseFolder() const
 {
     return QStringLiteral("/");
@@ -643,6 +555,12 @@ QPair<QString, QString> Theme::oauthOverrideAuthUrl() const
     return {};
 }
 
+QVector<quint16> Theme::oauthPorts() const
+{
+    // zero means a random port
+    return {0};
+}
+
 QString Theme::openIdConnectScopes() const
 {
     return QStringLiteral("openid offline_access email profile");
@@ -653,6 +571,11 @@ QString Theme::openIdConnectPrompt() const
     return QStringLiteral("select_account consent");
 }
 
+bool Theme::oidcEnableDynamicRegistration() const
+{
+    return true;
+}
+
 QString Theme::versionSwitchOutput() const
 {
     return aboutVersions(Theme::VersionFormat::Url);
@@ -693,5 +616,43 @@ bool Theme::allowDuplicatedFolderSyncPair() const
     return true;
 }
 
+bool Theme::wizardEnableWebfinger() const
+{
+    return false;
+}
+
+QVector<std::tuple<QString, QString, QUrl>> Theme::urlButtons() const
+{
+    return {};
+}
+
+bool Theme::enableCernBranding() const
+{
+    return false;
+}
+
+bool Theme::withCrashReporter() const
+{
+#ifdef WITH_CRASHREPORTER
+    return true;
+#else
+    return false;
+#endif
+}
+
+template <>
+OWNCLOUDSYNC_EXPORT QString Utility::enumToDisplayName(Theme::UserIDType userIdType)
+{
+    switch (userIdType) {
+    case Theme::UserIDUserName:
+        return QCoreApplication::translate("Type of user ID", "Username");
+    case Theme::UserIDEmail:
+        return QCoreApplication::translate("Type of user ID", "E-mail address");
+    case Theme::UserIDCustom:
+        return Theme::instance()->customUserID();
+    default:
+        Q_UNREACHABLE();
+    }
+}
 
 } // end namespace client
