@@ -1,6 +1,5 @@
 import json
 from urllib import parse
-from os import path
 
 from helpers.ConfigHelper import get_config
 from helpers.api.utils import url_join
@@ -28,7 +27,7 @@ def get_share_endpint():
 
 
 def create_space(space_name):
-    body = json.dumps({"Name": space_name})
+    body = json.dumps({"name": space_name})
     response = request.post(get_space_endpint(), body)
     if response.status_code != 201:
         raise Exception(
