@@ -7,60 +7,49 @@
 #pragma once
 
 #include "account.h"
-#include "creds/abstractcredentials.h"
-#include "logger.h"
-#include "filesystem.h"
-#include "folder.h"
-#include "syncengine.h"
+#include "common/filesystembase.h"
 #include "common/syncjournaldb.h"
 #include "common/syncjournalfilerecord.h"
 #include "common/vfs.h"
+#include "creds/abstractcredentials.h"
 #include "csync_exclude.h"
+#include "filesystem.h"
+#include "folder.h"
+#include "logger.h"
+#include "syncengine.h"
+#include "testutils.h"
 #include <cstring>
 
 #include <QDir>
-#include <QNetworkReply>
 #include <QMap>
+#include <QNetworkReply>
+#include <QTimer>
 #include <QtTest>
 #include <cookiejar.h>
-#include <QTimer>
 
+#include <chrono>
+
+using namespace OCC::FileSystem::SizeLiterals;
 /*
  * TODO: In theory we should use QVERIFY instead of Q_ASSERT for testing, but this
  * only works when directly called from a QTest :-(
  */
 
-
-static const QUrl sRootUrl = QUrl::fromEncoded("owncloud://somehost/owncloud/remote.php/webdav/");
-static const QUrl sRootUrl2 = QUrl::fromEncoded("owncloud://somehost/owncloud/remote.php/dav/files/admin/");
-static const QUrl sUploadUrl = QUrl::fromEncoded("owncloud://somehost/owncloud/remote.php/dav/uploads/admin/");
-
-inline QString getFilePathFromUrl(const QUrl &url)
-{
-    QString path = url.path();
-    if (path.startsWith(sRootUrl.path()))
-        return path.mid(sRootUrl.path().length());
-    if (path.startsWith(sRootUrl2.path()))
-        return path.mid(sRootUrl2.path().length());
-    if (path.startsWith(sUploadUrl.path()))
-        return path.mid(sUploadUrl.path().length());
-    return {};
-}
+QString getFilePathFromUrl(const QUrl &url);
 
 
 inline QByteArray generateEtag()
 {
-    return QByteArray::number(QDateTime::currentDateTimeUtc().toMSecsSinceEpoch(), 16) + QByteArray::number(qrand(), 16);
+    return QByteArray::number(QDateTime::currentDateTimeUtc().toMSecsSinceEpoch(), 16) + QByteArray::number(QRandomGenerator::global()->generate(), 16);
 }
 inline QByteArray generateFileId()
 {
-    return QByteArray::number(qrand(), 16);
+    return QByteArray::number(QRandomGenerator::global()->generate(), 16);
 }
 
 class PathComponents : public QStringList
 {
 public:
-    PathComponents(const char *path);
     PathComponents(const QString &path);
     PathComponents(const QStringList &pathComponents);
 
@@ -75,23 +64,54 @@ public:
     QString fileName() const { return last(); }
 };
 
+/**
+ * @brief The FileModifier class defines the interface for both the local on-disk modifier and the
+ * remote in-memory modifier.
+ */
 class FileModifier
 {
 public:
+    static constexpr auto DefaultFileSize = 64_b;
+    static constexpr char DefaultContentChar = 'X';
+
     virtual ~FileModifier() { }
     virtual void remove(const QString &relativePath) = 0;
-    virtual void insert(const QString &relativePath, qint64 size = 64, char contentChar = 'W') = 0;
-    virtual void setContents(const QString &relativePath, char contentChar) = 0;
-    virtual void appendByte(const QString &relativePath, char contentChar = 0) = 0;
-    virtual void modifyByte(const QString &relativePath, quint64 offset, char contentChar) = 0;
+    virtual void insert(const QString &relativePath, quint64 size = DefaultFileSize, char contentChar = DefaultContentChar) = 0;
+
+    virtual void setContents(const QString &relativePath, quint64 newSize, char contentChar = DefaultContentChar) = 0;
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void setContents(const QString &relativePath, T newSize, char contentChar = DefaultContentChar) = delete;
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void insert(const QString &relativePath, T size, char contentChar = DefaultContentChar) = delete;
+
+    virtual void appendByte(const QString &relativePath, char contentChar = DefaultContentChar) = 0;
     virtual void mkdir(const QString &relativePath) = 0;
     virtual void rename(const QString &relativePath, const QString &relativeDestinationDirectory) = 0;
     virtual void setModTime(const QString &relativePath, const QDateTime &modTime) = 0;
 };
 
+class FakeFolder;
+
+/**
+ * @brief The DiskFileModifier class is a FileModifier for files on the local disk.
+ *
+ * It uses a helper program (`test_helper`) to do the actual modifications. When running without a
+ * VFS, or when running with suffix-VFS, the changes are done before doing a sync. When running
+ * with a VFS, the OS will do callbacks to the test program. So in this case the modifications are
+ * done while the event loop is running, in order to process those OS callbacks.
+ *
+ * Note: these actions do NOT read content from files on disk: if the file would be dehydrated,
+ * reading will cause a re-hydration, which means network requests, and some tests explicitly check
+ * for the amount of requests.
+ */
 class DiskFileModifier : public FileModifier
 {
     QDir _rootDir;
+    QStringList _processArguments;
 
 public:
     DiskFileModifier(const QString &rootDirPath)
@@ -99,14 +119,23 @@ public:
     {
     }
     void remove(const QString &relativePath) override;
-    void insert(const QString &relativePath, qint64 size = 64, char contentChar = 'W') override;
-    void setContents(const QString &relativePath, char contentChar) override;
-    void appendByte(const QString &relativePath, char contentChar) override;
-    void modifyByte(const QString &relativePath, quint64 offset, char contentChar) override;
+    void insert(const QString &relativePath, quint64 size = DefaultFileSize, char contentChar = DefaultContentChar) override;
+    void setContents(const QString &relativePath, quint64 newSize, char contentChar = DefaultContentChar) override;
+    void appendByte(const QString &relativePath, char contentChar = DefaultContentChar) override;
 
     void mkdir(const QString &relativePath) override;
     void rename(const QString &from, const QString &to) override;
     void setModTime(const QString &relativePath, const QDateTime &modTime) override;
+
+    Q_REQUIRED_RESULT bool applyModifications();
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void setContents(const QString &relativePath, T newSize, char contentChar = DefaultContentChar) = delete;
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void insert(const QString &relativePath, T size, char contentChar = DefaultContentChar) = delete;
 };
 
 static inline qint64 defaultLastModified()
@@ -116,9 +145,17 @@ static inline qint64 defaultLastModified()
     return timeInSeconds;
 }
 
-/// FIXME: we should make it explicit in the construtor if we're talking about a hydrated or a dehydrated file!
+/**
+ * @brief The FileInfo class represents the remotely stored (on-server) content. To be able to
+ * modify the content, this class is also the remote \c FileModifier.
+ */
 class FileInfo : public FileModifier
 {
+    /// FIXME: we should make it explicit in the construtor if we're talking about a hydrated or a dehydrated file!
+    /// FIXME: this class is both a remote folder, as the root folder which in turn is the remote FileModifier.
+    ///        This is a mess: unifying remote files/folders is ok, but because it's also the remote root (which
+    ///        should implement the FileModifier), this means that each single remote file/folder also implements
+    ///        this interface.
 public:
     static FileInfo A12_B12_C12_S12();
 
@@ -127,14 +164,14 @@ public:
         : name { name }
     {
     }
-    FileInfo(const QString &name, qint64 size)
+    FileInfo(const QString &name, quint64 size)
         : name { name }
         , isDir { false }
         , fileSize(size)
         , contentSize { size }
     {
     }
-    FileInfo(const QString &name, qint64 size, char contentChar)
+    FileInfo(const QString &name, quint64 size, char contentChar)
         : name { name }
         , isDir { false }
         , fileSize(size)
@@ -148,13 +185,11 @@ public:
 
     void remove(const QString &relativePath) override;
 
-    void insert(const QString &relativePath, qint64 size = 64, char contentChar = 'W') override;
+    void insert(const QString &relativePath, quint64 size = DefaultFileSize, char contentChar = DefaultContentChar) override;
 
-    void setContents(const QString &relativePath, char contentChar) override;
+    void setContents(const QString &relativePath, quint64 newSize, char contentChar = DefaultContentChar) override;
 
-    void appendByte(const QString &relativePath, char contentChar = 0) override;
-
-    void modifyByte(const QString &relativePath, quint64 offset, char contentChar) override;
+    void appendByte(const QString &relativePath, char contentChar = DefaultContentChar) override;
 
     void mkdir(const QString &relativePath) override;
 
@@ -164,10 +199,11 @@ public:
 
     /// Return a pointer to the FileInfo, or a nullptr if it doesn't exist
     FileInfo *find(PathComponents pathComponents, const bool invalidateEtags = false);
+    FileInfo *find(const QString &pathComponents, const bool invalidateEtags = false) { return find(PathComponents{pathComponents}, invalidateEtags); }
 
     FileInfo *createDir(const QString &relativePath);
 
-    FileInfo *create(const QString &relativePath, qint64 size, char contentChar);
+    FileInfo *create(const QString &relativePath, quint64 size, char contentChar);
 
     bool operator<(const FileInfo &other) const
     {
@@ -227,8 +263,8 @@ public:
     QByteArray fileId = generateFileId();
     QByteArray checksums;
     QByteArray extraDavProperties;
-    qint64 fileSize = 0;
-    qint64 contentSize = 0;
+    quint64 fileSize = 0;
+    quint64 contentSize = 0;
     char contentChar = 'W';
     bool isDehydratedPlaceholder = false;
 
@@ -251,14 +287,23 @@ public:
             << ", children:" << fi.children
             << " }";
     }
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void setContents(const QString &relativePath, T newSize, char contentChar = DefaultContentChar) = delete;
+
+    // prevent implicit cast to quint64
+    template <typename T>
+    void insert(const QString &relativePath, T size, char contentChar = DefaultContentChar) = delete;
 };
 
 class FakeReply : public QNetworkReply
 {
     Q_OBJECT
+
 public:
     FakeReply(QObject *parent);
-    virtual ~FakeReply();
+    virtual ~FakeReply() override;
 
     // useful to be public for testing
     using QNetworkReply::setRawHeader;
@@ -267,6 +312,7 @@ public:
 class FakePropfindReply : public FakeReply
 {
     Q_OBJECT
+
 public:
     QByteArray payload;
 
@@ -285,7 +331,6 @@ public:
 class FakePutReply : public FakeReply
 {
     Q_OBJECT
-    FileInfo *fileInfo;
 
 public:
     FakePutReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, const QByteArray &putPayload, QObject *parent);
@@ -296,12 +341,14 @@ public:
 
     void abort() override;
     qint64 readData(char *, qint64) override { return 0; }
+
+private:
+    FileInfo *fileInfo;
 };
 
 class FakeMkcolReply : public FakeReply
 {
     Q_OBJECT
-    FileInfo *fileInfo;
 
 public:
     FakeMkcolReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
@@ -310,11 +357,15 @@ public:
 
     void abort() override { }
     qint64 readData(char *, qint64) override { return 0; }
+
+private:
+    FileInfo *fileInfo;
 };
 
 class FakeDeleteReply : public FakeReply
 {
     Q_OBJECT
+
 public:
     FakeDeleteReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
 
@@ -327,6 +378,7 @@ public:
 class FakeMoveReply : public FakeReply
 {
     Q_OBJECT
+
 public:
     FakeMoveReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
 
@@ -339,39 +391,30 @@ public:
 class FakeGetReply : public FakeReply
 {
     Q_OBJECT
+
 public:
+    enum class State {
+        Ok,
+        Aborted,
+        FileNotFound,
+    };
+
     const FileInfo *fileInfo;
     char payload;
-    int size;
-    bool aborted = false;
+    qint64 size;
+    State state = State::Ok;
+    const std::pair<qint64, qint64> _range;
 
     FakeGetReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
 
     Q_INVOKABLE void respond();
 
     void abort() override;
-    qint64 bytesAvailable() const override;
+    virtual qint64 bytesAvailable() const override;
 
-    qint64 readData(char *data, qint64 maxlen) override;
-};
+    virtual qint64 readData(char *data, qint64 maxlen) override;
 
-class FakeGetWithDataReply : public FakeReply
-{
-    Q_OBJECT
-public:
-    const FileInfo *fileInfo;
-    QByteArray payload;
-    quint64 offset = 0;
-    bool aborted = false;
-
-    FakeGetWithDataReply(FileInfo &remoteRootFileInfo, const QByteArray &data, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
-
-    Q_INVOKABLE void respond();
-
-    void abort() override;
-    qint64 bytesAvailable() const override;
-
-    qint64 readData(char *data, qint64 maxlen) override;
+    static std::pair<qint64, qint64> parseRange(const QNetworkRequest &request);
 };
 
 class FakeChunkMoveReply : public FakeReply
@@ -452,12 +495,12 @@ class DelayedReply : public OriginalReply
 {
 public:
     template <typename... Args>
-    explicit DelayedReply(quint64 delayMS, Args &&...args)
+    explicit DelayedReply(const std::chrono::milliseconds delayMS, Args &&...args)
         : OriginalReply(std::forward<Args>(args)...)
         , _delayMs(delayMS)
     {
     }
-    quint64 _delayMs;
+    std::chrono::milliseconds _delayMs;
 
     void respond() override
     {
@@ -468,7 +511,7 @@ public:
     }
 };
 
-class FakeQNAM : public QNetworkAccessManager
+class FakeAM : public OCC::AccessManager
 {
 public:
     using Override = std::function<QNetworkReply *(Operation, const QNetworkRequest &, QIODevice *)>;
@@ -482,7 +525,7 @@ private:
     Override _override;
 
 public:
-    FakeQNAM(FileInfo initialRoot);
+    FakeAM(FileInfo initialRoot, QObject *parent);
     FileInfo &currentRemoteState() { return _remoteRootFileInfo; }
     FileInfo &uploadState() { return _uploadFileInfo; }
 
@@ -497,16 +540,15 @@ protected:
 
 class FakeCredentials : public OCC::AbstractCredentials
 {
-    QNetworkAccessManager *_qnam;
-
 public:
-    FakeCredentials(QNetworkAccessManager *qnam)
-        : _qnam { qnam }
+    FakeCredentials(OCC::AccessManager *am)
+        : _am { am }
     {
     }
+
     QString authType() const override { return QStringLiteral("test"); }
     QString user() const override { return QStringLiteral("admin"); }
-    QNetworkAccessManager *createQNAM() const override { return _qnam; }
+    OCC::AccessManager *createAM() const override { return _am; }
     bool ready() const override { return true; }
     void fetchFromKeychain() override { }
     void askFromUser() override { }
@@ -514,47 +556,51 @@ public:
     void persist() override { }
     void invalidateToken() override { }
     void forgetSensitiveData() override { }
+
+private:
+    OCC::AccessManager *_am;
 };
 
-class FakeFolder
+class FakeFolder : public QObject
 {
-    QTemporaryDir _tempDir;
+    Q_OBJECT
+    const QTemporaryDir _tempDir = OCC::TestUtils::createTempDir();
     DiskFileModifier _localModifier;
-    // FIXME: Clarify ownership, double delete
-    FakeQNAM *_fakeQnam;
-    OCC::AccountPtr _account;
+    FakeAM *_fakeAm;
+    OCC::TestUtils::TestUtilsPrivate::AccountStateRaii _accountState =
+        OCC::TestUtils::TestUtilsPrivate::AccountStateRaii{nullptr, &OCC::TestUtils::TestUtilsPrivate::accountStateDeleter};
     std::unique_ptr<OCC::SyncJournalDb> _journalDb;
     std::unique_ptr<OCC::SyncEngine> _syncEngine;
-    OCC::Vfs::Mode _vfsMode;
 
 public:
-    FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode = OCC::Vfs::Off);
+    FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode = OCC::Vfs::Off, bool filesAreDehydrated = false);
+    ~FakeFolder();
 
     void switchToVfs(QSharedPointer<OCC::Vfs> vfs);
 
-    OCC::AccountPtr account() const { return _account; }
+    OCC::AccountPtr account() const { return _accountState->account(); }
     OCC::SyncEngine &syncEngine() const { return *_syncEngine; }
     OCC::SyncJournalDb &syncJournal() const { return *_journalDb; }
 
     FileModifier &localModifier() { return _localModifier; }
-    FileInfo &remoteModifier() { return _fakeQnam->currentRemoteState(); }
+    FileInfo &remoteModifier() { return _fakeAm->currentRemoteState(); }
     FileInfo currentLocalState();
 
-    FileInfo currentRemoteState() { return _fakeQnam->currentRemoteState(); }
-    FileInfo &uploadState() { return _fakeQnam->uploadState(); }
+    FileInfo &currentRemoteState() { return _fakeAm->currentRemoteState(); }
+    FileInfo &uploadState() { return _fakeAm->uploadState(); }
     FileInfo dbState() const;
 
     struct ErrorList
     {
-        FakeQNAM *_qnam;
+        FakeAM *_qnam;
         void append(const QString &path, int error = 500)
         {
             _qnam->errorPaths().insert(path, error);
         }
         void clear() { _qnam->errorPaths().clear(); }
     };
-    ErrorList serverErrorPaths() { return { _fakeQnam }; }
-    void setServerOverride(const FakeQNAM::Override &override) { _fakeQnam->setOverride(override); }
+    ErrorList serverErrorPaths() { return { _fakeAm }; }
+    void setServerOverride(const FakeAM::Override &override) { _fakeAm->setOverride(override); }
 
     QString localPath() const;
 
@@ -564,21 +610,25 @@ public:
 
     void execUntilItemCompleted(const QString &relativePath);
 
-    bool execUntilFinished()
+    bool execUntilFinished();
+
+    bool syncOnce();
+
+    Q_REQUIRED_RESULT bool applyLocalModificationsAndSync()
     {
-        QSignalSpy spy(_syncEngine.get(), &OCC::SyncEngine::finished);
-        bool ok = spy.wait(3600000);
-        Q_ASSERT(ok && "Sync timed out");
-        return spy[0][0].toBool();
+        if (!_localModifier.applyModifications()) {
+            return false;
+        }
+        return syncOnce();
     }
 
-    bool syncOnce()
+    Q_REQUIRED_RESULT bool applyLocalModificationsWithoutSync()
     {
-        scheduleSync();
-        return execUntilFinished();
+        return _localModifier.applyModifications();
     }
 
     bool isDehydratedPlaceholder(const QString &filePath);
+    QSharedPointer<OCC::Vfs> vfs() const;
 
 private:
     static void toDisk(QDir &dir, const FileInfo &templateFi);
@@ -625,8 +675,8 @@ inline void addFiles(QStringList &dest, const FileInfo &fi)
 {
     if (fi.isDir) {
         dest += QStringLiteral("%1 - dir").arg(fi.path());
-        for (const auto &fi : fi.children)
-            addFiles(dest, fi);
+        for (const auto &child : fi.children)
+            addFiles(dest, child);
     } else {
         dest += QStringLiteral("%1 - %2 %3-bytes (%4)").arg(fi.path(), QString::number(fi.contentSize), QChar::fromLatin1(fi.contentChar), fi.lastModifiedInUtc().toString());
     }
@@ -635,8 +685,9 @@ inline void addFiles(QStringList &dest, const FileInfo &fi)
 inline QString toStringNoElide(const FileInfo &fi)
 {
     QStringList files;
-    for (const auto &fi : fi.children)
-        addFiles(files, fi);
+    for (const auto &childInfo : fi.children) {
+        addFiles(files, childInfo);
+    }
     files.sort();
     return QStringLiteral("FileInfo with %1 files(\n\t%2\n)").arg(files.size()).arg(files.join(QStringLiteral("\n\t")));
 }
@@ -651,8 +702,8 @@ inline void addFilesDbData(QStringList &dest, const FileInfo &fi)
     // could include etag, permissions etc, but would need extra work
     if (fi.isDir) {
         dest += QStringLiteral("%1 - %2 %3 %4").arg(fi.name, fi.isDir ? QStringLiteral("dir") : QStringLiteral("file"), QString::number(fi.lastModifiedInSecondsUTC()), QString::fromUtf8(fi.fileId));
-        for (const auto &fi : fi.children)
-            addFilesDbData(dest, fi);
+        for (const auto &child : fi.children)
+            addFilesDbData(dest, child);
     } else {
         dest += QStringLiteral("%1 - %2 %3 %4 %5").arg(fi.name, fi.isDir ? QStringLiteral("dir") : QStringLiteral("file"), QString::number(fi.contentSize), QString::number(fi.lastModifiedInSecondsUTC()), QString::fromUtf8(fi.fileId));
     }
@@ -661,7 +712,57 @@ inline void addFilesDbData(QStringList &dest, const FileInfo &fi)
 inline char *printDbData(const FileInfo &fi)
 {
     QStringList files;
-    for (const auto &fi : fi.children)
-        addFilesDbData(files, fi);
+    for (const auto &child : fi.children)
+        addFilesDbData(files, child);
     return QTest::toString(QStringLiteral("FileInfo with %1 files(%2)").arg(files.size()).arg(files.join(QStringLiteral(", "))));
 }
+
+struct OperationCounter
+{
+    int nGET = 0;
+    int nPUT = 0;
+    int nMOVE = 0;
+    int nDELETE = 0;
+
+    OperationCounter() {};
+    OperationCounter(const OperationCounter &) = delete;
+    OperationCounter(OperationCounter &&) = delete;
+    void operator=(OperationCounter const &) = delete;
+    void operator=(OperationCounter &&) = delete;
+
+    void reset()
+    {
+        nGET = 0;
+        nPUT = 0;
+        nMOVE = 0;
+        nDELETE = 0;
+    }
+
+    QNetworkReply *serverOverride(QNetworkAccessManager::Operation op, const QNetworkRequest &req, QIODevice *)
+    {
+        if (op == QNetworkAccessManager::GetOperation) {
+            ++nGET;
+        } else if (op == QNetworkAccessManager::PutOperation) {
+            ++nPUT;
+        } else if (op == QNetworkAccessManager::DeleteOperation) {
+            ++nDELETE;
+        } else if (req.attribute(QNetworkRequest::CustomVerbAttribute).toByteArray() == QByteArrayLiteral("MOVE")) {
+            ++nMOVE;
+        }
+        return nullptr;
+    }
+    auto functor()
+    {
+        return [this](QNetworkAccessManager::Operation op, const QNetworkRequest &req, QIODevice *device) { return serverOverride(op, req, device); };
+    }
+
+    OperationCounter(FakeFolder &fakeFolder)
+    {
+        fakeFolder.setServerOverride(functor());
+    }
+
+    friend inline QDebug operator<<(QDebug dbg, const OperationCounter &oc)
+    {
+        return dbg << "nGET:" << oc.nGET << " nPUT:" << oc.nPUT << " nMOVE:" << oc.nMOVE << " nDELETE:" << oc.nDELETE;
+    }
+};
